@@ -2,6 +2,8 @@
 #   define TESS_CAPI_INCLUDE_BASEAPI
 #endif
 #include "capi.h"
+#include "genericvector.h"
+#include "strngs.h"
 
 TESS_API const char* TESS_CALL TessVersion()
 {
@@ -30,6 +32,96 @@ TESS_API void TESS_CALL TessDeleteBlockList(BLOCK_LIST* block_list)
     TessBaseAPI::DeleteBlockList(block_list);
 }
 
+TESS_API TessResultRenderer* TESS_CALL TessTextRendererCreate()
+{
+    return new TessTextRenderer;
+}
+
+TESS_API TessResultRenderer* TESS_CALL TessHOcrRendererCreate()
+{
+    return new TessHOcrRenderer;
+}
+
+TESS_API TessResultRenderer* TESS_CALL TessPDFRendererCreate(const char* datadir)
+{
+    return new TessPDFRenderer(datadir);
+}
+
+TESS_API TessResultRenderer* TESS_CALL TessUnlvRendererCreate()
+{
+    return new TessUnlvRenderer;
+}
+
+TESS_API TessResultRenderer* TESS_CALL TessBoxTextRendererCreate()
+{
+    return new TessBoxTextRenderer;
+}
+
+TESS_API void TESS_CALL TessDeleteResultRenderer(TessResultRenderer* renderer)
+{
+    delete [] renderer;
+}
+
+TESS_API void TESS_CALL TessResultRendererInsert(TessResultRenderer* renderer, TessResultRenderer* next)
+{
+    renderer->insert(next);
+}
+
+TESS_API TessResultRenderer* TESS_CALL TessResultRendererNext(TessResultRenderer* renderer)
+{
+    return renderer->next();
+}
+
+TESS_API BOOL TESS_CALL TessResultRendererBeginDocument(TessResultRenderer* renderer, const char* title)
+{
+    return renderer->BeginDocument(title);
+}
+
+TESS_API BOOL TESS_CALL TessResultRendererAddImage(TessResultRenderer* renderer, TessBaseAPI* api)
+{
+    return renderer->AddImage(api);
+}
+
+TESS_API BOOL TESS_CALL TessResultRendererAddError(TessResultRenderer* renderer, TessBaseAPI* api)
+{
+    return renderer->AddError(api);
+}
+
+TESS_API BOOL TESS_CALL TessResultRendererEndDocument(TessResultRenderer* renderer)
+{
+    return renderer->EndDocument();
+}
+
+TESS_API BOOL TESS_CALL TessResultRendererGetOutput(TessResultRenderer* renderer, const char** data, int* data_len)
+{
+    const char* tmp_data;
+    int tmp_data_len;
+    bool boolValue = renderer->GetOutput(&tmp_data, &tmp_data_len);
+    *data = tmp_data;
+    *data_len = tmp_data_len;
+    return boolValue;
+}
+
+TESS_API const char* TESS_CALL TessResultRendererTypename(TessResultRenderer* renderer)
+{
+    return renderer->full_typename();
+}
+
+TESS_API const char* TESS_CALL TessResultRendererExtention(TessResultRenderer* renderer)
+{
+    return renderer->file_extension();
+}
+
+TESS_API const char* TESS_CALL TessResultRendererTitle(TessResultRenderer* renderer)
+{
+    return renderer->title();
+}
+
+TESS_API int TESS_CALL TessResultRendererImageNum(TessResultRenderer* renderer)
+{
+    return renderer->imagenum();
+}
+
 TESS_API TessBaseAPI* TESS_CALL TessBaseAPICreate()
 {
     return new TessBaseAPI;
@@ -40,11 +132,41 @@ TESS_API void TESS_CALL TessBaseAPIDelete(TessBaseAPI* handle)
     delete handle;
 }
 
+TESS_API size_t TESS_CALL TessBaseAPIGetOpenCLDevice(TessBaseAPI* handle, void **device)
+{
+    return handle->getOpenCLDevice(device);
+}
+
 TESS_API void TESS_CALL TessBaseAPISetInputName(TessBaseAPI* handle, const char* name)
 {
     handle->SetInputName(name);
 }
 
+TESS_API const char* TESS_CALL TessBaseAPIGetInputName(TessBaseAPI* handle)
+{
+    return handle->GetInputName();
+}
+
+TESS_API void TESS_CALL TessBaseAPISetInputImage(TessBaseAPI* handle, Pix* pix)
+{
+    handle->SetInputImage(pix);
+}
+
+TESS_API Pix* TESS_CALL TessBaseAPIGetInputImage(TessBaseAPI* handle)
+{
+    return handle->GetInputImage();
+}
+
+TESS_API int TESS_CALL TessBaseAPIGetSourceYResolution(TessBaseAPI* handle)
+{
+    return handle->GetSourceYResolution();
+}
+
+TESS_API const char* TESS_CALL TessBaseAPIGetDatapath(TessBaseAPI* handle)
+{
+    return handle->GetDatapath();
+}
+
 TESS_API void TESS_CALL TessBaseAPISetOutputName(TessBaseAPI* handle, const char* name)
 {
     handle->SetOutputName(name);
@@ -111,6 +233,24 @@ TESS_API BOOL TESS_CALL TessBaseAPIGetVariableAsString(TessBaseAPI* handle, cons
     return handle->GetVariableAsString(name, val) ? TRUE : FALSE;
 }
 
+TESS_API int TESS_CALL TessBaseAPIInit4(TessBaseAPI* handle, const char* datapath, const char* language,
+    TessOcrEngineMode mode, char** configs, int configs_size,
+    char** vars_vec, char** vars_values, size_t vars_vec_size,
+    BOOL set_only_non_debug_params)
+{
+    GenericVector<STRING> varNames;
+    GenericVector<STRING> varValues;
+    if (vars_vec != NULL && vars_values != NULL) {
+        for (size_t i = 0; i < vars_vec_size; i++) {
+            varNames.push_back(STRING(vars_vec[i]));
+            varValues.push_back(STRING(vars_values[i]));
+        }
+    }
+
+    return handle->Init(datapath, language, mode, configs, configs_size, &varNames, &varValues, set_only_non_debug_params);
+}
+
+
 TESS_API int TESS_CALL TessBaseAPIInit1(TessBaseAPI* handle, const char* datapath, const char* language, TessOcrEngineMode oem,
                                         char** configs, int configs_size)
 {
@@ -202,7 +342,7 @@ TESS_API void TESS_CALL TessBaseAPISetImage(TessBaseAPI* handle, const unsigned
     handle->SetImage(imagedata, width, height, bytes_per_pixel, bytes_per_line);
 }
 
-TESS_API void TESS_CALL TessBaseAPISetImage2(TessBaseAPI* handle, const Pix* pix)
+TESS_API void TESS_CALL TessBaseAPISetImage2(TessBaseAPI* handle, const struct Pix* pix)
 {
     return handle->SetImage(pix);
 }
@@ -222,41 +362,55 @@ TESS_API void TESS_CALL TessBaseAPISetThresholder(TessBaseAPI* handle, TessImage
     handle->SetThresholder(thresholder);
 }
 
-TESS_API Pix* TESS_CALL TessBaseAPIGetThresholdedImage(TessBaseAPI* handle)
+TESS_API struct Pix* TESS_CALL TessBaseAPIGetThresholdedImage(TessBaseAPI* handle)
 {
     return handle->GetThresholdedImage();
 }
 
-TESS_API Boxa* TESS_CALL TessBaseAPIGetRegions(TessBaseAPI* handle, Pixa** pixa)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetRegions(TessBaseAPI* handle, struct Pixa** pixa)
 {
     return handle->GetRegions(pixa);
 }
 
-TESS_API Boxa* TESS_CALL TessBaseAPIGetTextlines(TessBaseAPI* handle, Pixa** pixa, int** blockids)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetTextlines(TessBaseAPI* handle, struct Pixa** pixa, int** blockids)
 {
     return handle->GetTextlines(pixa, blockids);
 }
 
-TESS_API BOXA* TESS_CALL TessBaseAPIGetStrips(TessBaseAPI* handle, PIXA** pixa, int** blockids)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetTextlines1(TessBaseAPI* handle, const BOOL raw_image, const int raw_padding,
+                                                                  struct Pixa** pixa, int** blockids, int** paraids)
+{
+    return handle->GetTextlines(raw_image, raw_padding, pixa, blockids, paraids);
+}
+
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetStrips(TessBaseAPI* handle, struct Pixa** pixa, int** blockids)
 {
     return handle->GetStrips(pixa, blockids);
 }
 
-TESS_API Boxa* TESS_CALL TessBaseAPIGetWords(TessBaseAPI* handle, Pixa** pixa)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetWords(TessBaseAPI* handle, struct Pixa** pixa)
 {
     return handle->GetWords(pixa);
 }
 
-TESS_API Boxa* TESS_CALL TessBaseAPIGetConnectedComponents(TessBaseAPI* handle, Pixa** cc)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetConnectedComponents(TessBaseAPI* handle, struct Pixa** cc)
 {
     return handle->GetConnectedComponents(cc);
 }
 
-TESS_API Boxa* TESS_CALL TessBaseAPIGetComponentImages(TessBaseAPI* handle, TessPageIteratorLevel level, BOOL text_only, Pixa** pixa, int** blockids)
+TESS_API struct Boxa* TESS_CALL TessBaseAPIGetComponentImages(TessBaseAPI* handle, TessPageIteratorLevel level, BOOL text_only, struct Pixa** pixa, int** blockids)
 {
     return handle->GetComponentImages(level, text_only != FALSE, pixa, blockids);
 }
 
+TESS_API struct Boxa*
+               TESS_CALL TessBaseAPIGetComponentImages1(   TessBaseAPI* handle, const TessPageIteratorLevel level, const BOOL text_only,
+                                                           const BOOL raw_image, const int raw_padding,
+                                                           struct Pixa** pixa, int** blockids, int** paraids)
+{
+    return handle->GetComponentImages(level, text_only != FALSE, raw_image, raw_padding, pixa, blockids, paraids);
+}
+
 TESS_API int TESS_CALL TessBaseAPIGetThresholdedImageScaleFactor(const TessBaseAPI* handle)
 {
     return handle->GetThresholdedImageScaleFactor();
@@ -292,16 +446,34 @@ TESS_API char* TESS_CALL TessBaseAPIProcessPages(TessBaseAPI* handle, const char
         return NULL;
 }
 
-TESS_API char* TESS_CALL TessBaseAPIProcessPage(TessBaseAPI* handle, Pix* pix, int page_index, const char* filename,
+TESS_API BOOL TessBaseAPIProcessPages1(TessBaseAPI* handle, const char* filename, const char* retry_config,
+                                                 int timeout_millisec, TessResultRenderer* renderer)
+{
+    if (handle->ProcessPages(filename, retry_config, timeout_millisec, renderer))    
+        return TRUE;
+    else
+        return FALSE;
+}
+
+TESS_API char* TESS_CALL TessBaseAPIProcessPage(TessBaseAPI* handle, struct Pix* pix, int page_index, const char* filename,
                                                 const char* retry_config, int timeout_millisec)
 {
     STRING text_out;
     if (handle->ProcessPage(pix, page_index, filename, retry_config, timeout_millisec, &text_out))
-        return text_out.strdup();    
+        return text_out.strdup();
     else
         return NULL;
 }
 
+TESS_API BOOL TessBaseAPIProcessPage1(TessBaseAPI* handle, struct Pix* pix, int page_index, const char* filename,
+                                      const char* retry_config, int timeout_millisec, TessResultRenderer* renderer)
+{
+    if (handle->ProcessPage(pix, page_index, filename, retry_config, timeout_millisec, renderer))
+        return TRUE;
+    else
+        return FALSE;
+}
+
 TESS_API TessResultIterator* TESS_CALL TessBaseAPIGetIterator(TessBaseAPI* handle)
 {
     return handle->GetIterator();
@@ -357,7 +529,7 @@ TESS_API void TESS_CALL TessBaseAPIEnd(TessBaseAPI* handle)
     handle->End();
 }
 
-TESS_API int TESS_CALL TessBaseAPIIsValidWord(TessBaseAPI* handle, const char *word)
+TESS_API int TESS_CALL TessBaseAPIIsValidWord(TessBaseAPI* handle, const char* word)
 {
     return handle->IsValidWord(word);
 }
@@ -372,6 +544,11 @@ TESS_API void TESS_CALL TessBaseAPISetDictFunc(TessBaseAPI* handle, TessDictFunc
     handle->SetDictFunc(f);
 }
 
+TESS_API void  TESS_CALL TessBaseAPIClearPersistentCache(TessBaseAPI* handle)
+{
+    handle->ClearPersistentCache();
+}
+
 TESS_API void TESS_CALL TessBaseAPISetProbabilityInContextFunc(TessBaseAPI* handle, TessProbabilityInContextFunc f)
 {
     handle->SetProbabilityInContextFunc(f);
@@ -382,10 +559,10 @@ TESS_API BOOL TESS_CALL TessBaseAPIDetectOS(TessBaseAPI* handle, OSResults* resu
     return handle->DetectOS(results) ? TRUE : FALSE;
 }
 
-TESS_API void TESS_CALL TessBaseAPIGetFeaturesForBlob(TessBaseAPI* handle, TBLOB* blob, const DENORM* denorm, INT_FEATURE_ARRAY int_features,
+TESS_API void TESS_CALL TessBaseAPIGetFeaturesForBlob(TessBaseAPI* handle, TBLOB* blob, INT_FEATURE_STRUCT* int_features,
                                                             int* num_features, int* FeatureOutlineIndex)
 {
-    handle->GetFeaturesForBlob(blob, *denorm, int_features, num_features, FeatureOutlineIndex);
+    handle->GetFeaturesForBlob(blob, int_features, num_features, FeatureOutlineIndex);
 }
 
 TESS_API ROW* TESS_CALL TessFindRowForBox(BLOCK_LIST* blocks, int left, int top, int right, int bottom)
@@ -393,10 +570,10 @@ TESS_API ROW* TESS_CALL TessFindRowForBox(BLOCK_LIST* blocks, int left, int top,
     return TessBaseAPI::FindRowForBox(blocks, left, top, right, bottom);
 }
 
-TESS_API void TESS_CALL TessBaseAPIRunAdaptiveClassifier(TessBaseAPI* handle, TBLOB* blob, const DENORM* denorm, int num_max_matches,
+TESS_API void TESS_CALL TessBaseAPIRunAdaptiveClassifier(TessBaseAPI* handle, TBLOB* blob, int num_max_matches,
                                                                int* unichar_ids, float* ratings, int* num_matches_returned)
 {
-    handle->RunAdaptiveClassifier(blob, *denorm, num_max_matches, unichar_ids, ratings, num_matches_returned);
+    handle->RunAdaptiveClassifier(blob, num_max_matches, unichar_ids, ratings, num_matches_returned);
 }
 
 TESS_API const char* TESS_CALL TessBaseAPIGetUnichar(TessBaseAPI* handle, int unichar_id)
@@ -419,14 +596,14 @@ TESS_API ROW* TESS_CALL TessMakeTessOCRRow(float baseline, float xheight, float
     return TessBaseAPI::MakeTessOCRRow(baseline, xheight, descender, ascender);
 }
 
-TESS_API TBLOB* TESS_CALL TessMakeTBLOB(Pix *pix)
+TESS_API TBLOB* TESS_CALL TessMakeTBLOB(struct Pix* pix)
 {
     return TessBaseAPI::MakeTBLOB(pix);
 }
 
-TESS_API void TESS_CALL TessNormalizeTBLOB(TBLOB *tblob, ROW *row, BOOL numeric_mode, DENORM *denorm)
+TESS_API void TESS_CALL TessNormalizeTBLOB(TBLOB* tblob, ROW* row, BOOL numeric_mode)
 {
-    TessBaseAPI::NormalizeTBLOB(tblob, row, numeric_mode != FALSE, denorm);
+    TessBaseAPI::NormalizeTBLOB(tblob, row, numeric_mode != FALSE);
 }
 
 TESS_API TessOcrEngineMode TESS_CALL TessBaseAPIOem(const TessBaseAPI* handle)
@@ -434,7 +611,7 @@ TESS_API TessOcrEngineMode TESS_CALL TessBaseAPIOem(const TessBaseAPI* handle)
     return handle->oem();
 }
 
-TESS_API void TESS_CALL TessBaseAPIInitTruthCallback(TessBaseAPI* handle, TessTruthCallback *cb)
+TESS_API void TESS_CALL TessBaseAPIInitTruthCallback(TessBaseAPI* handle, TessTruthCallback* cb)
 {
     handle->InitTruthCallback(cb);
 }
@@ -501,12 +678,12 @@ TESS_API TessPolyBlockType TESS_CALL TessPageIteratorBlockType(const TessPageIte
     return handle->BlockType();
 }
 
-TESS_API Pix* TESS_CALL TessPageIteratorGetBinaryImage(const TessPageIterator* handle, TessPageIteratorLevel level)
+TESS_API struct Pix* TESS_CALL TessPageIteratorGetBinaryImage(const TessPageIterator* handle, TessPageIteratorLevel level)
 {
     return handle->GetBinaryImage(level);
 }
 
-TESS_API Pix* TESS_CALL TessPageIteratorGetImage(const TessPageIterator* handle, TessPageIteratorLevel level, int padding,
+TESS_API struct Pix* TESS_CALL TessPageIteratorGetImage(const TessPageIterator* handle, TessPageIteratorLevel level, int padding,
                                                        int* left, int* top)
 {
     return handle->GetImage(level, padding, left, top);
@@ -518,9 +695,9 @@ TESS_API BOOL TESS_CALL TessPageIteratorBaseline(const TessPageIterator* handle,
     return handle->Baseline(level, x1, y1, x2, y2) ? TRUE : FALSE;
 }
 
-TESS_API void TESS_CALL TessPageIteratorOrientation(TessPageIterator* handle, TessOrientation *orientation,
-                                                          TessWritingDirection *writing_direction, TessTextlineOrder *textline_order,
-                                                          float *deskew_angle)
+TESS_API void TESS_CALL TessPageIteratorOrientation(TessPageIterator* handle, TessOrientation* orientation,
+                                                          TessWritingDirection* writing_direction, TessTextlineOrder* textline_order,
+                                                          float* deskew_angle)
 {
     handle->Orientation(orientation, writing_direction, textline_order, deskew_angle);
 }
@@ -532,7 +709,7 @@ TESS_API void TESS_CALL TessResultIteratorDelete(TessResultIterator* handle)
 
 TESS_API TessResultIterator* TESS_CALL TessResultIteratorCopy(const TessResultIterator* handle)
 {
-    return static_cast<TessResultIterator*>(new TessPageIterator(*handle));
+    return new TessResultIterator(*handle);
 }
 
 TESS_API TessPageIterator* TESS_CALL TessResultIteratorGetPageIterator(TessResultIterator* handle)
