@@ -21,7 +21,6 @@
 #pragma warning(disable:4244)  // Conversion warnings
 #endif
 
-#include "mfcpch.h"
 #include <string.h>
 #include <ctype.h>
 #ifdef __UNIX__
@@ -30,14 +29,11 @@
 #include          <errno.h>
 #endif
 #include "helpers.h"
-#include "tfacep.h"
 #include "tessvars.h"
 #include "control.h"
-#include "secname.h"
 #include "reject.h"
 #include "docqual.h"
 #include "output.h"
-#include "bestfirst.h"
 #include "globals.h"
 #include "tesseractclass.h"
 
@@ -73,7 +69,6 @@ void Tesseract::output_pass(  //Tess output pass //send to api
                             PAGE_RES_IT &page_res_it,
                             const TBOX *target_word_box) {
   BLOCK_RES *block_of_last_word;
-  inT16 block_id;
   BOOL8 force_eol;               //During output
   BLOCK *nextblock;              //block of next word
   WERD *nextword;                //next word
@@ -98,7 +93,6 @@ void Tesseract::output_pass(  //Tess output pass //send to api
     if (tessedit_write_block_separators &&
     block_of_last_word != page_res_it.block ()) {
       block_of_last_word = page_res_it.block ();
-      block_id = block_of_last_word->block->index();
     }
 
     force_eol = (tessedit_write_block_separators &&
@@ -140,18 +134,10 @@ void Tesseract::write_results(PAGE_RES_IT &page_res_it,
                               BOOL8 force_eol) {  // override tilde crunch?
   WERD_RES *word = page_res_it.word();
   const UNICHARSET &uchset = *word->uch_set;
-  STRING repetition_code;
-  const STRING *wordstr;
-  STRING wordstr_lengths;
   int i;
-  char unrecognised = STRING (unrecognised_char)[0];
-  char ep_chars[32];             //Only for unlv_tilde_crunch
-  int ep_chars_index = 0;
-  char txt_chs[32];              //Only for unlv_tilde_crunch
-  char map_chs[32];              //Only for unlv_tilde_crunch
-  int txt_index = 0;
   BOOL8 need_reject = FALSE;
   UNICHAR_ID space = uchset.unichar_to_id(" ");
+
   if ((word->unlv_crunch_mode != CR_NONE ||
        word->best_choice->length() == 0) &&
       !tessedit_zero_kelvin_rejection && !tessedit_word_for_word) {
@@ -165,10 +151,6 @@ void Tesseract::write_results(PAGE_RES_IT &page_res_it,
           (word->word->space () > 0) &&
           !word->word->flag (W_FUZZY_NON) &&
           !word->word->flag (W_FUZZY_SP)) {
-        // Write a space to separate from preceeding good text.
-        txt_chs[txt_index] = ' ';
-        map_chs[txt_index++] = '1';
-        ep_chars[ep_chars_index++] = ' ';
         stats_.last_char_was_tilde = false;
       }
       need_reject = TRUE;
@@ -177,50 +159,16 @@ void Tesseract::write_results(PAGE_RES_IT &page_res_it,
         (force_eol && stats_.write_results_empty_block)) {
       /* Write a reject char - mark as rejected unless zero_rejection mode */
       stats_.last_char_was_tilde = TRUE;
-      txt_chs[txt_index] = unrecognised;
-      if (tessedit_zero_rejection || (suspect_level == 0)) {
-        map_chs[txt_index++] = '1';
-        ep_chars[ep_chars_index++] = unrecognised;
-      }
-      else {
-        map_chs[txt_index++] = '0';
-        /*
-           The ep_choice string is a faked reject to allow newdiff to sync the
-           .etx with the .txt and .map files.
-         */
-        ep_chars[ep_chars_index++] = CTRL_INSET; // escape code
-                                 //dummy reject
-        ep_chars[ep_chars_index++] = 1;
-                                 //dummy reject
-        ep_chars[ep_chars_index++] = 1;
-                                 //type
-        ep_chars[ep_chars_index++] = 2;
-                                 //dummy reject
-        ep_chars[ep_chars_index++] = 1;
-                                 //dummy reject
-        ep_chars[ep_chars_index++] = 1;
-      }
       stats_.tilde_crunch_written = true;
       stats_.last_char_was_newline = false;
       stats_.write_results_empty_block = false;
     }
 
     if ((word->word->flag (W_EOL) && !stats_.last_char_was_newline) || force_eol) {
-      /* Add a new line output */
-      txt_chs[txt_index] = '\n';
-      map_chs[txt_index++] = '\n';
-                                 //end line
-      ep_chars[ep_chars_index++] = newline_type;
-
-                                 //Cos of the real newline
       stats_.tilde_crunch_written = false;
       stats_.last_char_was_newline = true;
       stats_.last_char_was_tilde = false;
     }
-    txt_chs[txt_index] = '\0';
-    map_chs[txt_index] = '\0';
-    ep_chars[ep_chars_index] = '\0';  // terminate string
-    word->ep_choice = new WERD_CHOICE(ep_chars, uchset);
 
     if (force_eol)
       stats_.write_results_empty_block = true;
@@ -243,13 +191,7 @@ void Tesseract::write_results(PAGE_RES_IT &page_res_it,
       (word->best_choice->unichar_id(0) == space)) {
     /* Prevent adjacent tilde across words - we know that adjacent tildes within
        words have been removed */
-    word->best_choice->remove_unichar_id(0);
-    if (word->best_choice->blob_choices() != NULL) {
-      BLOB_CHOICE_LIST_C_IT blob_choices_it(word->best_choice->blob_choices());
-      if (!blob_choices_it.empty()) delete blob_choices_it.extract();
-    }
-    word->reject_map.remove_pos (0);
-    word->box_word->DeleteBox(0);
+    word->MergeAdjacentBlobs(0);
   }
   if (newline_type ||
     (word->word->flag (W_REP_CHAR) && tessedit_write_rep_codes))
@@ -275,13 +217,7 @@ void Tesseract::write_results(PAGE_RES_IT &page_res_it,
              word->best_choice->debug_string().string(),
              dict_word(*(word->best_choice)));
   }
-  if (word->word->flag (W_REP_CHAR) && tessedit_write_rep_codes) {
-    repetition_code = "|^~R";
-    wordstr_lengths = "\001\001\001\001";
-    repetition_code += uchset.id_to_unichar(get_rep_char(word));
-    wordstr_lengths += strlen(uchset.id_to_unichar(get_rep_char(word)));
-    wordstr = &repetition_code;
-  } else {
+  if (!word->word->flag(W_REP_CHAR) || !tessedit_write_rep_codes) {
     if (tessedit_zero_rejection) {
       /* OVERRIDE ALL REJECTION MECHANISMS - ONLY REJECT TESS FAILURES */
       for (i = 0; i < word->best_choice->length(); ++i) {
