@@ -204,6 +204,10 @@ class BLOBNBOX:public ELIST_LINK
     // given horizontal range.
     TBOX BoundsWithinLimits(int left, int right);
 
+    // Estimates and stores the baseline position based on the shape of the
+    // outline.
+    void EstimateBaselinePosition();
+
     // Simple accessors.
     const TBOX& bounding_box() const {
       return box;
@@ -219,6 +223,7 @@ class BLOBNBOX:public ELIST_LINK
       box = cblob_ptr->bounding_box();
       base_char_top_ = box.top();
       base_char_bottom_ = box.bottom();
+      baseline_y_ = box.bottom();
     }
     const TBOX& reduced_box() const {
       return red_box;
@@ -363,6 +368,9 @@ class BLOBNBOX:public ELIST_LINK
     int base_char_bottom() const {
       return base_char_bottom_;
     }
+    int baseline_position() const {
+      return baseline_y_;
+    }
     int line_crossings() const {
       return line_crossings_;
     }
@@ -407,6 +415,10 @@ class BLOBNBOX:public ELIST_LINK
     static void CleanNeighbours(BLOBNBOX_LIST* blobs);
     // Helper to delete all the deletable blobs on the list.
     static void DeleteNoiseBlobs(BLOBNBOX_LIST* blobs);
+    // Helper to compute edge offsets for  all the blobs on the list.
+    // See coutln.h for an explanation of edge offsets.
+    static void ComputeEdgeOffsets(Pix* thresholds, Pix* grey,
+                                   BLOBNBOX_LIST* blobs);
 
 #ifndef GRAPHICS_DISABLED
     // Helper to draw all the blobs on the list in the given body_colour,
@@ -431,10 +443,7 @@ class BLOBNBOX:public ELIST_LINK
 
     void plot(ScrollView* window,                // window to draw in
               ScrollView::Color blob_colour,     // for outer bits
-              ScrollView::Color child_colour) {  // for holes
-      if (cblob_ptr != NULL)
-        cblob_ptr->plot(window, blob_colour, child_colour);
-    }
+              ScrollView::Color child_colour);   // for holes
 #endif
 
   // Initializes the bulk of the members to default values for use at
@@ -467,6 +476,7 @@ class BLOBNBOX:public ELIST_LINK
     owner_ = NULL;
     base_char_top_ = box.top();
     base_char_bottom_ = box.bottom();
+    baseline_y_ = box.bottom();
     line_crossings_ = 0;
     base_char_blob_ = NULL;
     horz_possible_ = false;
@@ -501,6 +511,7 @@ class BLOBNBOX:public ELIST_LINK
   inT16 right_crossing_rule_;   // x-coord of nearest or crossing rule line
   inT16 base_char_top_;         // y-coord of top/bottom of diacritic base,
   inT16 base_char_bottom_;      // if it exists else top/bottom of this blob.
+  inT16 baseline_y_;            // Estimate of baseline position.
   int line_crossings_;          // Number of line intersections touched.
   BLOBNBOX* base_char_blob_;    // The blob that was the base char.
   float horz_stroke_width_;     // Median horizontal stroke width
@@ -530,6 +541,7 @@ class TO_ROW: public ELIST2_LINK
            float bottom,
            float row_size);
 
+    void print() const;
     float max_y() const {  //access function
       return y_max;
     }
@@ -710,12 +722,12 @@ class TO_BLOCK:public ELIST_LINK
       TO_ROW_IT row_it = &row_list;
       TO_ROW *row;
 
-      for (row_it.mark_cycle_pt (); !row_it.cycled_list ();
-      row_it.forward ()) {
-        row = row_it.data ();
-        printf ("Row range (%g,%g), para_c=%g, blobcount=" INT32FORMAT
-          "\n", row->min_y (), row->max_y (), row->parallel_c (),
-          row->blob_list ()->length ());
+      for (row_it.mark_cycle_pt(); !row_it.cycled_list();
+           row_it.forward()) {
+        row = row_it.data();
+        tprintf("Row range (%g,%g), para_c=%g, blobcount=" INT32FORMAT
+                "\n", row->min_y(), row->max_y(), row->parallel_c(),
+                row->blob_list()->length());
       }
     }
 
@@ -728,6 +740,15 @@ class TO_BLOCK:public ELIST_LINK
     // Deletes noise blobs from all lists where not owned by a ColPartition.
     void DeleteUnownedNoise();
 
+    // Computes and stores the edge offsets on each blob for use in feature
+    // extraction, using greyscale if the supplied grey and thresholds pixes
+    // are 8-bit or otherwise (if NULL or not 8 bit) the original binary
+    // edge step outlines.
+    // Thresholds must either be the same size as grey or an integer down-scale
+    // of grey.
+    // See coutln.h for an explanation of edge offsets.
+    void ComputeEdgeOffsets(Pix* thresholds, Pix* grey);
+
 #ifndef GRAPHICS_DISABLED
     // Draw the noise blobs from all lists in red.
     void plot_noise_blobs(ScrollView* to_win);
