@@ -19,6 +19,7 @@
 #ifndef           PAGERES_H
 #define           PAGERES_H
 
+#include "blamer.h"
 #include "blobs.h"
 #include "boxword.h"
 #include "elst.h"
@@ -38,167 +39,6 @@ class Tesseract;
 }
 using tesseract::FontInfo;
 
-static const inT16 kBlamerBoxTolerance = 5;
-
-// Enum for expressing the source of error.
-// Note: Please update kIncorrectResultReasonNames when modifying this enum.
-enum IncorrectResultReason {
-  // The text recorded in best choice == truth text
-  IRR_CORRECT,
-  // Either: Top choice is incorrect and is a dictionary word (language model
-  // is unlikely to help correct such errors, so blame the classifier).
-  // Or: the correct unichar was not included in shortlist produced by the
-  // classifier at all.
-  IRR_CLASSIFIER,
-  // Chopper have not found one or more splits that correspond to the correct
-  // character bounding boxes recorded in BlamerBundle::truth_word.
-  IRR_CHOPPER,
-  // Classifier did include correct unichars for each blob in the correct
-  // segmentation, however its rating could have been too bad to allow the
-  // language model to pull out the correct choice. On the other hand the
-  // strength of the language model might have been too weak to favor the
-  // correct answer, this we call this case a classifier-language model
-  // tradeoff error.
-  IRR_CLASS_LM_TRADEOFF,
-  // Page layout failed to produce the correct bounding box. Blame page layout
-  // if the truth was not found for the word, which implies that the bounding
-  // box of the word was incorrect (no truth word had a similar bounding box).
-  IRR_PAGE_LAYOUT,
-  // SegSearch heuristic prevented one or more blobs from the correct
-  // segmentation state to be classified (e.g. the blob was too wide).
-  IRR_SEGSEARCH_HEUR,
-  // The correct segmentaiton state was not explored because of poor SegSearch
-  // pain point prioritization. We blame SegSearch pain point prioritization
-  // if the best rating of a choice constructed from correct segmentation is
-  // better than that of the best choice (i.e. if we got to explore the correct
-  // segmentation state, language model would have picked the correct choice).
-  IRR_SEGSEARCH_PP,
-  // Same as IRR_CLASS_LM_TRADEOFF, but used when we only run chopper on a word,
-
-  // and thus use the old language model (permuters).
-  // TODO(antonova): integrate the new language mode with chopper
-  IRR_CLASS_OLD_LM_TRADEOFF,
-  // If there is an incorrect adaptive template match with a better score than
-  // a correct one (either pre-trained or adapted), mark this as adaption error.
-  IRR_ADAPTION,
-  // split_and_recog_word() failed to find a suitable split in truth.
-  IRR_NO_TRUTH_SPLIT,
-  // Truth is not available for this word (e.g. when words in corrected content
-  // file are turned into ~~~~ because an appropriate alignment was not found.
-  IRR_NO_TRUTH,
-  // The text recorded in best choice != truth text, but none of the above
-  // reasons are set.
-  IRR_UNKNOWN,
-
-  IRR_NUM_REASONS
-};
-
-// Blamer-related information to determine the source of errors.
-struct BlamerBundle {
-  static const char *IncorrectReasonName(IncorrectResultReason irr);
-  BlamerBundle() : truth_has_char_boxes(false),
-      incorrect_result_reason(IRR_CORRECT),
-      lattice_data(NULL) { ClearResults(); }
-  ~BlamerBundle() { delete[] lattice_data; }
-  void ClearResults() {
-    norm_truth_word.DeleteAllBoxes();
-    norm_box_tolerance = 0;
-    if (!NoTruth()) incorrect_result_reason = IRR_CORRECT;
-    debug = "";
-    segsearch_is_looking_for_blame = false;
-    best_correctly_segmented_rating = WERD_CHOICE::kBadRating;
-    correct_segmentation_cols.clear();
-    correct_segmentation_rows.clear();
-    best_choice_is_dict_and_top_choice = false;
-    delete[] lattice_data;
-    lattice_data = NULL;
-    lattice_size = 0;
-  }
-  void CopyTruth(const BlamerBundle &other) {
-    truth_has_char_boxes = other.truth_has_char_boxes;
-    truth_word = other.truth_word;
-    truth_text = other.truth_text;
-    incorrect_result_reason =
-        (other.NoTruth() ? other.incorrect_result_reason : IRR_CORRECT);
-  }
-  void CopyResults(const BlamerBundle &other) {
-    norm_truth_word = other.norm_truth_word;
-    norm_box_tolerance = other.norm_box_tolerance;
-    incorrect_result_reason = other.incorrect_result_reason;
-    segsearch_is_looking_for_blame = other.segsearch_is_looking_for_blame;
-    best_correctly_segmented_rating =other.best_correctly_segmented_rating;
-    correct_segmentation_cols = other.correct_segmentation_cols;
-    correct_segmentation_rows = other.correct_segmentation_rows;
-    best_choice_is_dict_and_top_choice =
-        other.best_choice_is_dict_and_top_choice;
-    if (other.lattice_data != NULL) {
-      lattice_data = new char[other.lattice_size];
-      memcpy(lattice_data, other.lattice_data, other.lattice_size);
-      lattice_size = other.lattice_size;
-    } else {
-      lattice_data = NULL;
-    }
-  }
-  BlamerBundle(const BlamerBundle &other) {
-    this->CopyTruth(other);
-    this->CopyResults(other);
-  }
-  const char *IncorrectReason() const;
-  bool NoTruth() const {
-    return (incorrect_result_reason == IRR_NO_TRUTH ||
-             incorrect_result_reason == IRR_PAGE_LAYOUT);
-  }
-  void SetBlame(IncorrectResultReason irr,
-                const STRING &msg, const WERD_CHOICE *choice, bool debug) {
-    this->incorrect_result_reason = irr;
-    this->debug = this->IncorrectReason();
-    this->debug += " to blame: ";
-    this->FillDebugString(msg, choice, &(this->debug));
-    if (debug) tprintf("SetBlame(): %s", this->debug.string());
-  }
-  // Appends choice and truth details to the given debug string.
-  void FillDebugString(const STRING &msg, const WERD_CHOICE *choice,
-                       STRING *debug);
-
-  // Set to true when bounding boxes for individual unichars are recorded.
-  bool truth_has_char_boxes;
-  // The true_word (in the original image coordinate space) contains ground
-  // truth bounding boxes for this WERD_RES.
-  tesseract::BoxWord truth_word;
-  // Same as above, but in normalized coordinates
-  // (filled in by WERD_RES::SetupForRecognition()).
-  tesseract::BoxWord norm_truth_word;
-  // Tolerance for bounding box comparisons in normalized space.
-  int norm_box_tolerance;
-  // Contains ground truth unichar for each of the bounding boxes in truth_word.
-  GenericVector<STRING> truth_text;
-  // The reason for incorrect OCR result.
-  IncorrectResultReason incorrect_result_reason;
-  // Debug text associated with the blame.
-  STRING debug;
-  // Misadaption debug information (filled in if this word was misadapted to).
-  STRING misadaption_debug;
-  // Variables used by the segmentation search when looking for the blame.
-  // Set to true while segmentation search is continued after the usual
-  // termination condition in order to look for the blame.
-  bool segsearch_is_looking_for_blame;
-  // Best rating for correctly segmented path
-  // (set and used by SegSearch when looking for blame).
-  float best_correctly_segmented_rating;
-  // Vectors populated by SegSearch to indicate column and row indices that
-  // correspond to blobs with correct bounding boxes.
-  GenericVector<int> correct_segmentation_cols;
-  GenericVector<int> correct_segmentation_rows;
-  // Set to true if best choice is a dictionary word and
-  // classifier's top choice.
-  bool best_choice_is_dict_and_top_choice;
-  // Serialized segmentation search lattice.
-  char *lattice_data;
-  int lattice_size;  // size of lattice_data in bytes
-  // Information about hypotheses (paths) explored by the segmentation search.
-  tesseract::ParamsTrainingBundle params_training_bundle;
-};
-
 /* Forward declarations */
 
 class BLOCK_RES;
@@ -341,8 +181,11 @@ class WERD_RES : public ELIST_LINK {
   // TODO(rays) determine if docqual does anything useful and delete bln_boxes
   // if it doesn't.
   tesseract::BoxWord* bln_boxes;  // BLN input bounding boxes.
+  // The ROW that this word sits in. NOT owned by the WERD_RES.
+  ROW* blob_row;
   // The denorm provides the transformation to get back to the rotated image
-  // coords from the chopped_word/rebuild_word BLN coords.
+  // coords from the chopped_word/rebuild_word BLN coords, but each blob also
+  // has its own denorm.
   DENORM denorm;                  // For use on chopped_word.
   // Unicharset used by the classifier output in best_choice and raw_choice.
   const UNICHARSET* uch_set;  // For converting back to utf8.
@@ -355,13 +198,32 @@ class WERD_RES : public ELIST_LINK {
   // character fragments that make up the word.
   // The length of chopped_word matches length of seam_array + 1 (if set).
   TWERD* chopped_word;            // BLN chopped fragments output.
-  SEAMS seam_array;               // Seams matching chopped_word.
-  WERD_CHOICE *best_choice;       // tess output
-  WERD_CHOICE *raw_choice;        // top choice permuter
-  // Alternative paths found during chopping/segmentation search stages
-  // (the first entry being a slim copy of best_choice).
-  GenericVector<WERD_CHOICE *> alt_choices;
-  GenericVector<GenericVector<int> > alt_states;
+  // Vector of SEAM* holding chopping points matching chopped_word.
+  GenericVector<SEAM*> seam_array;
+  // Widths of blobs in chopped_word.
+  GenericVector<int> blob_widths;
+  // Gaps between blobs in chopped_word. blob_gaps[i] is the gap between
+  // blob i and blob i+1.
+  GenericVector<int> blob_gaps;
+  // Ratings matrix contains classifier choices for each classified combination
+  // of blobs. The dimension is the same as the number of blobs in chopped_word
+  // and the leading diagonal corresponds to classifier results of the blobs
+  // in chopped_word. The state_ members of best_choice, raw_choice and
+  // best_choices all correspond to this ratings matrix and allow extraction
+  // of the blob choices for any given WERD_CHOICE.
+  MATRIX* ratings;                // Owned pointer.
+  // Pointer to the first WERD_CHOICE in best_choices. This is the result that
+  // will be output from Tesseract. Note that this is now a borrowed pointer
+  // and should NOT be deleted.
+  WERD_CHOICE* best_choice;       // Borrowed pointer.
+  // The best raw_choice found during segmentation search. Differs from the
+  // best_choice by being the best result according to just the character
+  // classifier, not taking any language model information into account.
+  // Unlike best_choice, the pointer IS owned by this WERD_RES.
+  WERD_CHOICE* raw_choice;        // Owned pointer.
+  // Alternative results found during chopping/segmentation search stages.
+  // Note that being an ELIST, best_choices owns the WERD_CHOICEs.
+  WERD_CHOICE_LIST best_choices;
 
   // Truth bounding boxes, text and incorrect choice reason.
   BlamerBundle *blamer_bundle;
@@ -462,6 +324,8 @@ class WERD_RES : public ELIST_LINK {
     InitPointers();
     word = the_word;
   }
+  // Deep copies everything except the ratings MATRIX.
+  // To get that use deep_copy below.
   WERD_RES(const WERD_RES &source) {
     InitPointers();
     *this = source;            // see operator=
@@ -475,7 +339,8 @@ class WERD_RES : public ELIST_LINK {
   // characters purely based on their shape on the page, and by default produce
   // the corresponding unicode for a left-to-right context.
   const char* const BestUTF8(int blob_index, bool in_rtl_context) const {
-    if (blob_index < 0 || blob_index >= best_choice->length())
+    if (blob_index < 0 || best_choice == NULL ||
+        blob_index >= best_choice->length())
       return NULL;
     UNICHAR_ID id = best_choice->unichar_id(blob_index);
     if (id < 0 || id >= uch_set->size() || id == INVALID_UNICHAR_ID)
@@ -545,7 +410,11 @@ class WERD_RES : public ELIST_LINK {
   void InitPointers();
   void Clear();
   void ClearResults();
+  void ClearWordChoices();
+  void ClearRatings();
 
+  // Deep copies everything except the ratings MATRIX.
+  // To get that use deep_copy below.
   WERD_RES& operator=(const WERD_RES& source);  //from this
 
   void CopySimpleFields(const WERD_RES& source);
@@ -557,24 +426,31 @@ class WERD_RES : public ELIST_LINK {
   void InitForRetryRecognition(const WERD_RES& source);
 
   // Sets up the members used in recognition: bln_boxes, chopped_word,
-  // seam_array, denorm, best_choice, raw_choice.  Returns false if
+  // seam_array, denorm.  Returns false if
   // the word is empty and sets up fake results.  If use_body_size is
   // true and row->body_size is set, then body_size will be used for
   // blob normalization instead of xheight + ascrise. This flag is for
   // those languages that are using CJK pitch model and thus it has to
   // be true if and only if tesseract->textord_use_cjk_fp_model is
   // true.
-  bool SetupForTessRecognition(const UNICHARSET& unicharset_in,
-                               tesseract::Tesseract* tesseract, Pix* pix,
-                               bool numeric_mode, bool use_body_size,
-                               ROW *row, BLOCK* block);
-
-  // Sets up the members used in recognition:
-  // bln_boxes, chopped_word, seam_array, denorm.
+  // If allow_detailed_fx is true, the feature extractor will receive fine
+  // precision outline information, allowing smoother features and better
+  // features on low resolution images.
+  // The norm_mode sets the default mode for normalization in absence
+  // of any of the above flags. It should really be a tesseract::OcrEngineMode
+  // but is declared as int for ease of use with tessedit_ocr_engine_mode.
   // Returns false if the word is empty and sets up fake results.
-  bool SetupForCubeRecognition(const UNICHARSET& unicharset_in,
-                               tesseract::Tesseract* tesseract,
-                               const BLOCK* block);
+  bool SetupForRecognition(const UNICHARSET& unicharset_in,
+                           tesseract::Tesseract* tesseract, Pix* pix,
+                           int norm_mode,
+                           const TBOX* norm_box, bool numeric_mode,
+                           bool use_body_size, bool allow_detailed_fx,
+                           ROW *row, const BLOCK* block);
+
+  // Set up the seam array, bln_boxes, best_choice, and raw_choice to empty
+  // accumulators from a made chopped word.  We presume the fields are already
+  // empty.
+  void SetupBasicsFromChoppedWord(const UNICHARSET &unicharset_in);
 
   // Sets up the members used in recognition for an empty recognition result:
   // bln_boxes, chopped_word, seam_array, denorm, best_choice, raw_choice.
@@ -586,6 +462,87 @@ class WERD_RES : public ELIST_LINK {
   // Sets up the blamer_bundle if it is not null, using the initialized denorm.
   void SetupBlamerBundle();
 
+  // Computes the blob_widths and blob_gaps from the chopped_word.
+  void SetupBlobWidthsAndGaps();
+
+  // Updates internal data to account for a new SEAM (chop) at the given
+  // blob_number. Fixes the ratings matrix and states in the choices, as well
+  // as the blob widths and gaps.
+  void InsertSeam(int blob_number, SEAM* seam);
+
+  // Returns true if all the word choices except the first have adjust_factors
+  // worse than the given threshold.
+  bool AlternativeChoiceAdjustmentsWorseThan(float threshold) const;
+
+  // Returns true if the current word is ambiguous (by number of answers or
+  // by dangerous ambigs.)
+  bool IsAmbiguous();
+
+  // Returns true if the ratings matrix size matches the sum of each of the
+  // segmentation states.
+  bool StatesAllValid();
+
+  // Prints a list of words found if debug is true or the word result matches
+  // the word_to_debug.
+  void DebugWordChoices(bool debug, const char* word_to_debug);
+
+  // Removes from best_choices all choices which are not within a reasonable
+  // range of the best choice.
+  void FilterWordChoices(int debug_level);
+
+  // Computes a set of distance thresholds used to control adaption.
+  // Compares the best choice for the current word to the best raw choice
+  // to determine which characters were classified incorrectly by the
+  // classifier. Then places a separate threshold into thresholds for each
+  // character in the word. If the classifier was correct, max_rating is placed
+  // into thresholds. If the classifier was incorrect, the mean match rating
+  // (error percentage) of the classifier's incorrect choice minus some margin
+  // is placed into thresholds. This can then be used by the caller to try to
+  // create a new template for the desired class that will classify the
+  // character with a rating better than the threshold value. The match rating
+  // placed into thresholds is never allowed to be below min_rating in order to
+  // prevent trying to make overly tight templates.
+  // min_rating limits how tight to make a template.
+  // max_rating limits how loose to make a template.
+  // rating_margin denotes the amount of margin to put in template.
+  void ComputeAdaptionThresholds(float certainty_scale,
+                                 float min_rating,
+                                 float max_rating,
+                                 float rating_margin,
+                                 float* thresholds);
+
+  // Saves a copy of the word_choice if it has the best unadjusted rating.
+  // Returns true if the word_choice was the new best.
+  bool LogNewRawChoice(WERD_CHOICE* word_choice);
+  // Consumes word_choice by adding it to best_choices, (taking ownership) if
+  // the certainty for word_choice is some distance of the best choice in
+  // best_choices, or by deleting the word_choice and returning false.
+  // The best_choices list is kept in sorted order by rating. Duplicates are
+  // removed, and the list is kept no longer than max_num_choices in length.
+  // Returns true if the word_choice is still a valid pointer.
+  bool LogNewCookedChoice(int max_num_choices, bool debug,
+                          WERD_CHOICE* word_choice);
+
+  // Prints a brief list of all the best choices.
+  void PrintBestChoices() const;
+
+  // Returns the sum of the widths of the blob between start_blob and last_blob
+  // inclusive.
+  int GetBlobsWidth(int start_blob, int last_blob);
+  // Returns the width of a gap between the specified blob and the next one.
+  int GetBlobsGap(int blob_index);
+
+  // Returns the BLOB_CHOICE corresponding to the given index in the
+  // best choice word taken from the appropriate cell in the ratings MATRIX.
+  // Borrowed pointer, so do not delete. May return NULL if there is no
+  // BLOB_CHOICE matching the unichar_id at the given index.
+  BLOB_CHOICE* GetBlobChoice(int index) const;
+
+  // Returns the BLOB_CHOICE_LIST corresponding to the given index in the
+  // best choice word taken from the appropriate cell in the ratings MATRIX.
+  // Borrowed pointer, so do not delete.
+  BLOB_CHOICE_LIST* GetBlobChoices(int index) const;
+
   // Moves the results fields from word to this. This takes ownership of all
   // the data, so src can be destructed.
   // word1.ConsumeWordResult(word);
@@ -597,10 +554,11 @@ class WERD_RES : public ELIST_LINK {
   void ConsumeWordResults(WERD_RES* word);
 
   // Replace the best choice and rebuild box word.
-  void ReplaceBestChoice(const WERD_CHOICE& choice,
-                         const GenericVector<int> &segmentation_state);
+  // choice must be from the current best_choices list.
+  void ReplaceBestChoice(WERD_CHOICE* choice);
 
-  // Builds the rebuild_word from the chopped_word and the best_state.
+  // Builds the rebuild_word and sets the best_state from the chopped_word and
+  // the best_choice->state.
   void RebuildBestState();
 
   // Copies the chopped_word to the rebuild_word, faking a best_state as well.
@@ -610,30 +568,26 @@ class WERD_RES : public ELIST_LINK {
   // Sets/replaces the box_word with one made from the rebuild_word.
   void SetupBoxWord();
 
-  // Sets up the script positions in the output boxword using the best_choice
+  // Sets up the script positions in the best_choice using the best_choice
   // to get the unichars, and the unicharset to get the target positions.
   void SetScriptPositions();
-
-  // Returns the indices [start, end) containing the core of the word, stripped
-  // of any superscript digits on either side.
-  // (i.e., the non-footnote part of the word).
-  // Assumes that BoxWord is all set up for best_choice.
-  void WithoutFootnoteSpan(int *start, int *end) const;
-
-  // Given an alternate word choice and segmentation state, yield the indices
-  // [start, end) containig the core of the word, stripped of any superscript
-  // digits on either side.  (i.e. stripping off the footnote parts).
-  void WithoutFootnoteSpan(
-      const WERD_CHOICE &choice, const GenericVector<int> &state,
-      int *start, int *end) const;
+  // Sets all the blobs in all the words (best choice and alternates) to be
+  // the given position. (When a sub/superscript is recognized as a separate
+  // word, it falls victim to the rule that a whole word cannot be sub or
+  // superscript, so this function overrides that problem.)
+  void SetAllScriptPositions(tesseract::ScriptPos position);
 
   // Classifies the word with some already-calculated BLOB_CHOICEs.
   // The choices are an array of blob_count pointers to BLOB_CHOICE,
   // providing a single classifier result for each blob.
   // The BLOB_CHOICEs are consumed and the word takes ownership.
-  // The number of blobs in the outword must match blob_count.
+  // The number of blobs in the box_word must match blob_count.
   void FakeClassifyWord(int blob_count, BLOB_CHOICE** choices);
 
+  // Creates a WERD_CHOICE for the word using the top choices from the leading
+  // diagonal of the ratings matrix.
+  void FakeWordFromRatings();
+
   // Copies the best_choice strings to the correct_text for adaption/training.
   void BestChoiceToCorrectText();
 
@@ -644,13 +598,16 @@ class WERD_RES : public ELIST_LINK {
   // Returns true if anything was merged.
   bool ConditionalBlobMerge(
       TessResultCallback2<UNICHAR_ID, UNICHAR_ID, UNICHAR_ID>* class_cb,
-      TessResultCallback2<bool, const TBOX&, const TBOX&>* box_cb,
-      BLOB_CHOICE_LIST_CLIST *blob_choices);
+      TessResultCallback2<bool, const TBOX&, const TBOX&>* box_cb);
+
+  // Merges 2 adjacent blobs in the result (index and index+1) and corrects
+  // all the data to account for the change.
+  void MergeAdjacentBlobs(int index);
 
   // Callback helper for fix_quotes returns a double quote if both
   // arguments are quote, otherwise INVALID_UNICHAR_ID.
   UNICHAR_ID BothQuotes(UNICHAR_ID id1, UNICHAR_ID id2);
-  void fix_quotes(BLOB_CHOICE_LIST_CLIST *blob_choices);
+  void fix_quotes();
 
   // Callback helper for fix_hyphens returns UNICHAR_ID of - if both
   // arguments are hyphen, otherwise INVALID_UNICHAR_ID.
@@ -658,15 +615,21 @@ class WERD_RES : public ELIST_LINK {
   // Callback helper for fix_hyphens returns true if box1 and box2 overlap
   // (assuming both on the same textline, are in order and a chopped em dash.)
   bool HyphenBoxesOverlap(const TBOX& box1, const TBOX& box2);
-  void fix_hyphens(BLOB_CHOICE_LIST_CLIST *blob_choices);
+  void fix_hyphens();
 
   // Callback helper for merge_tess_fails returns a space if both
   // arguments are space, otherwise INVALID_UNICHAR_ID.
   UNICHAR_ID BothSpaces(UNICHAR_ID id1, UNICHAR_ID id2);
   void merge_tess_fails();
 
+  // Returns a really deep copy of *src, including the ratings MATRIX.
   static WERD_RES* deep_copy(const WERD_RES* src) {
-    return new WERD_RES(*src);
+    WERD_RES* result = new WERD_RES(*src);
+    // That didn't copy the ratings, but we want a copy if there is one to
+    // begin width.
+    if (src->ratings != NULL)
+      result->ratings = src->ratings->DeepCopy();
+    return result;
   }
 
   // Copy blobs from word_res onto this word (eliminating spaces between).
