@@ -20,25 +20,25 @@
 #ifndef           QUSPLINE_H
 #define           QUSPLINE_H
 
-#include          "grphics.h"
 #include          "quadratc.h"
 #include          "serialis.h"
 #include          "memry.h"
 #include          "rect.h"
 
 class ROW;
+struct Pix;
 
 class QSPLINE
 {
-  friend void make_first_baseline(BOX *,
+  friend void make_first_baseline(TBOX *,
                                   int,
                                   int *,
                                   int *,
                                   QSPLINE *,
                                   QSPLINE *,
                                   float);
-  friend void make_holed_baseline(BOX *, int, QSPLINE *, QSPLINE *, float); 
-  friend void tweak_row_baseline(ROW *); 
+  friend void make_holed_baseline(TBOX *, int, QSPLINE *, QSPLINE *, float);
+  friend void tweak_row_baseline(ROW *, double, double);
   public:
     QSPLINE() {  //empty constructor
       segments = 0;
@@ -48,8 +48,8 @@ class QSPLINE
     QSPLINE(  //copy constructor
             const QSPLINE &src);
     QSPLINE(                  //constructor
-            INT32 count,      //number of segments
-            INT32 *xstarts,   //segment starts
+            inT32 count,      //number of segments
+            inT32 *xstarts,   //segment starts
             double *coeffs);  //coefficients
     ~QSPLINE ();                 //destructor
     QSPLINE (                    //least squares fit
@@ -77,38 +77,23 @@ class QSPLINE
 
 #ifndef GRAPHICS_DISABLED
     void plot(                       //draw it
-              WINDOW window,         //in window
-              COLOUR colour) const;  //in colour
+              ScrollView* window,         //in window
+              ScrollView::Color colour) const;  //in colour
 #endif
 
-    void prep_serialise() {  //set ptrs to counts
-    }                            //not required
+    // Paint the baseline over pix. If pix has depth of 32, then the line will
+    // be painted in red. Otherwise it will be painted in black.
+    void plot(Pix* pix) const;
 
-    void dump(  //write external bits
-              FILE *f) {
-      serialise_bytes (f, (void *) xcoords, (segments + 1) * sizeof (INT32));
-      serialise_bytes (f, (void *) quadratics, segments * sizeof (QUAD_COEFFS));
-    }
-
-    void de_dump(  //read external bits
-                 FILE *f) {
-      xcoords = (INT32 *) de_serialise_bytes (f,
-        (segments + 1) * sizeof (INT32));
-      quadratics = (QUAD_COEFFS *) de_serialise_bytes (f,
-        segments *
-        sizeof (QUAD_COEFFS));
-    }
-
-                                 //assign copy
-    make_serialise (QSPLINE) QSPLINE & operator= (
+    QSPLINE & operator= (
       const QSPLINE & source);   //from this
 
   private:
 
-    INT32 spline_index(                  //binary search
+    inT32 spline_index(                  //binary search
                        double x) const;  //for x
-    INT32 segments;              //no of segments
-    INT32 *xcoords;              //no of coords
+    inT32 segments;              //no of segments
+    inT32 *xcoords;              //no of coords
     QUAD_COEFFS *quadratics;     //spline pieces
 };
 #endif
