@@ -20,6 +20,7 @@
 #ifndef TESSERACT_CCUTIL_GENERICVECTOR_H_
 #define TESSERACT_CCUTIL_GENERICVECTOR_H_
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -27,6 +28,7 @@
 #include "errcode.h"
 #include "helpers.h"
 #include "ndminx.h"
+#include "strngs.h"
 
 // Use PointerVector<T> below in preference to GenericVector<T*>, as that
 // provides automatic deletion of pointers, [De]Serialize that works, and
@@ -34,8 +36,13 @@
 template <typename T>
 class GenericVector {
  public:
-  GenericVector() { this->init(kDefaultVectorSize); }
-  explicit GenericVector(int size) { this->init(size); }
+  GenericVector() {
+    init(kDefaultVectorSize);
+  }
+  GenericVector(int size, T init_val) {
+    init(size);
+    init_to_size(size, init_val);
+  }
 
   // Copy
   GenericVector(const GenericVector& other) {
@@ -45,7 +52,7 @@ class GenericVector {
   GenericVector<T> &operator+=(const GenericVector& other);
   GenericVector<T> &operator=(const GenericVector& other);
 
-  virtual ~GenericVector();
+  ~GenericVector();
 
   // Reserve some memory.
   void reserve(int size);
@@ -59,6 +66,9 @@ class GenericVector {
   int size() const {
     return size_used_;
   }
+  int size_reserved() const {
+    return size_reserved_;
+  }
 
   int length() const {
     return size_used_;
@@ -73,6 +83,8 @@ class GenericVector {
   T &get(int index) const;
   T &back() const;
   T &operator[](int index) const;
+  // Returns the last object and removes it.
+  T pop_back();
 
   // Return the index of the T object.
   // This method NEEDS a compare_callback to be passed to
@@ -105,11 +117,11 @@ class GenericVector {
 
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
-  virtual void remove(int index);
+  void remove(int index);
 
   // Truncates the array to the given size by removing the end.
   // If the current size is less, the array is not expanded.
-  virtual void truncate(int size) {
+  void truncate(int size) {
     if (size < size_used_)
       size_used_ = size;
   }
@@ -126,7 +138,7 @@ class GenericVector {
   // All the owned callbacks are also deleted.
   // If you don't want the callbacks to be deleted, before calling clear, set
   // the callback to NULL.
-  virtual void clear();
+  void clear();
 
   // Delete objects pointed to by data_[i]
   void delete_data_pointers();
@@ -147,12 +159,12 @@ class GenericVector {
   bool read(FILE* f, TessResultCallback3<bool, FILE*, T*, bool>* cb, bool swap);
   // Writes a vector of simple types to the given file. Assumes that bitwise
   // read/write of T will work. Returns false in case of error.
-  virtual bool Serialize(FILE* fp) const;
+  bool Serialize(FILE* fp) const;
   // Reads a vector of simple types from the given file. Assumes that bitwise
   // read/write will work with ReverseN according to sizeof(T).
   // Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
-  virtual bool DeSerialize(bool swap, FILE* fp);
+  bool DeSerialize(bool swap, FILE* fp);
   // Writes a vector of classes to the given file. Assumes the existence of
   // bool T::Serialize(FILE* fp) const that returns false in case of error.
   // Returns false in case of error.
@@ -262,7 +274,32 @@ class GenericVector {
     return result;
   }
 
+  // Returns the index of what would be the target_index_th item in the array
+  // if the members were sorted, without actually sorting. Members are
+  // shuffled around, but it takes O(n) time.
+  // NOTE: uses operator< and operator== on the members.
+  int choose_nth_item(int target_index) {
+    // Make sure target_index is legal.
+    if (target_index < 0)
+      target_index = 0;                   // ensure legal
+    else if (target_index >= size_used_)
+      target_index = size_used_ - 1;
+    unsigned int seed = 1;
+    return choose_nth_item(target_index, 0, size_used_, &seed);
+  }
+
+  // Swaps the elements with the given indices.
+  void swap(int index1, int index2) {
+    if (index1 != index2) {
+      T tmp = data_[index1];
+      data_[index1] = data_[index2];
+      data_[index2] = tmp;
+    }
+  }
+
  protected:
+  // Internal recursive version of choose_nth_item.
+  int choose_nth_item(int target_index, int start, int end, unsigned int* seed);
 
   // Init the object, allocating size memory.
   void init(int size);
@@ -281,6 +318,40 @@ class GenericVector {
 
 namespace tesseract {
 
+// Function to read a GenericVector<char> from a whole file.
+// Returns false on failure.
+typedef bool (*FileReader)(const STRING& filename, GenericVector<char>* data);
+// Function to write a GenericVector<char> to a whole file.
+// Returns false on failure.
+typedef bool (*FileWriter)(const GenericVector<char>& data,
+                           const STRING& filename);
+// The default FileReader loads the whole file into the vector of char,
+// returning false on error.
+inline bool LoadDataFromFile(const STRING& filename,
+                             GenericVector<char>* data) {
+  FILE* fp = fopen(filename.string(), "rb");
+  if (fp == NULL) return false;
+  fseek(fp, 0, SEEK_END);
+  size_t size = ftell(fp);
+  fseek(fp, 0, SEEK_SET);
+  // Pad with a 0, just in case we treat the result as a string.
+  data->init_to_size(size + 1, 0);
+  bool result = fread(&(*data)[0], 1, size, fp) == size;
+  fclose(fp);
+  return result;
+}
+// The default FileWriter writes the vector of char to the filename file,
+// returning false on error.
+inline bool SaveDataToFile(const GenericVector<char>& data,
+                          const STRING& filename) {
+  FILE* fp = fopen(filename.string(), "wb");
+  if (fp == NULL) return false;
+  bool result =
+      static_cast<int>(fwrite(&data[0], 1, data.size(), fp)) == data.size();
+  fclose(fp);
+  return result;
+}
+
 template <typename T>
 bool cmp_eq(T const & t1, T const & t2) {
   return t1 == t2;
@@ -328,7 +399,7 @@ class PointerVector : public GenericVector<T*> {
  public:
   PointerVector() : GenericVector<T*>() { }
   explicit PointerVector(int size) : GenericVector<T*>(size) { }
-  virtual ~PointerVector() {
+  ~PointerVector() {
     // Clear must be called here, even though it is called again by the base,
     // as the base will call the wrong clear.
     clear();
@@ -355,14 +426,14 @@ class PointerVector : public GenericVector<T*> {
 
   // Removes an element at the given index and
   // shifts the remaining elements to the left.
-  virtual void remove(int index) {
+  void remove(int index) {
     delete GenericVector<T*>::data_[index];
     GenericVector<T*>::remove(index);
   }
 
   // Truncates the array to the given size by removing the end.
   // If the current size is less, the array is not expanded.
-  virtual void truncate(int size) {
+  void truncate(int size) {
     for (int i = size; i < GenericVector<T*>::size_used_; ++i)
       delete GenericVector<T*>::data_[i];
     GenericVector<T*>::truncate(size);
@@ -394,14 +465,14 @@ class PointerVector : public GenericVector<T*> {
   // All the owned callbacks are also deleted.
   // If you don't want the callbacks to be deleted, before calling clear, set
   // the callback to NULL.
-  virtual void clear() {
+  void clear() {
     GenericVector<T*>::delete_data_pointers();
     GenericVector<T*>::clear();
   }
 
   // Writes a vector of simple types to the given file. Assumes that bitwise
   // read/write of T will work. Returns false in case of error.
-  virtual bool Serialize(FILE* fp) const {
+  bool Serialize(FILE* fp) const {
     inT32 used = GenericVector<T*>::size_used_;
     if (fwrite(&used, sizeof(used), 1, fp) != 1) return false;
     for (int i = 0; i < used; ++i) {
@@ -416,7 +487,7 @@ class PointerVector : public GenericVector<T*> {
   // Also needs T::T(), as new T is used in this function.
   // Returns false in case of error.
   // If swap is true, assumes a big/little-endian swap is needed.
-  virtual bool DeSerialize(bool swap, FILE* fp) {
+  bool DeSerialize(bool swap, FILE* fp) {
     inT32 reserved;
     if (fread(&reserved, sizeof(reserved), 1, fp) != 1) return false;
     if (swap) Reverse32(&reserved);
@@ -427,9 +498,15 @@ class PointerVector : public GenericVector<T*> {
       T* item = NULL;
       if (non_null) {
         item = new T;
-        if (!item->DeSerialize(swap, fp)) return false;
+        if (!item->DeSerialize(swap, fp)) {
+          delete item;
+          return false;
+        }
+        this->push_back(item);
+      } else {
+        // Null elements should keep their place in the vector.
+        this->push_back(NULL);
       }
-      this->push_back(item);
     }
     return true;
   }
@@ -515,7 +592,8 @@ T &GenericVector<T>::get(int index) const {
 
 template <typename T>
 T &GenericVector<T>::operator[](int index) const {
- return data_[index];
+  assert(index >= 0 && index < size_used_);
+  return data_[index];
 }
 
 template <typename T>
@@ -523,6 +601,12 @@ T &GenericVector<T>::back() const {
   ASSERT_HOST(size_used_ > 0);
   return data_[size_used_ - 1];
 }
+// Returns the last object and removes it.
+template <typename T>
+T GenericVector<T>::pop_back() {
+  ASSERT_HOST(size_used_ > 0);
+  return data_[--size_used_];
+}
 
 // Return the object from an index.
 template <typename T>
@@ -536,7 +620,7 @@ void GenericVector<T>::set(T t, int index) {
 // at the specified index.
 template <typename T>
 void GenericVector<T>::insert(T t, int index) {
-  ASSERT_HOST(index >= 0 && index < size_used_);
+  ASSERT_HOST(index >= 0 && index <= size_used_);
   if (size_reserved_ == size_used_)
     double_the_size();
   for (int i = size_used_; i > index; --i) {
@@ -642,7 +726,8 @@ void GenericVector<T>::set_clear_callback(TessCallback1<T>* cb) {
 // Add a callback to be called to delete the elements when the array took
 // their ownership.
 template <typename T>
-void GenericVector<T>::set_compare_callback(TessResultCallback2<bool, T const &, T const &>* cb) {
+void GenericVector<T>::set_compare_callback(
+    TessResultCallback2<bool, T const &, T const &>* cb) {
   compare_cb_ = cb;
 }
 
@@ -804,4 +889,65 @@ void GenericVector<T>::sort() {
   sort(&tesseract::sort_cmp<T>);
 }
 
+// Internal recursive version of choose_nth_item.
+// The algorithm used comes from "Algorithms" by Sedgewick:
+// http://books.google.com/books/about/Algorithms.html?id=idUdqdDXqnAC
+// The principle is to choose a random pivot, and move everything less than
+// the pivot to its left, and everything greater than the pivot to the end
+// of the array, then recurse on the part that contains the desired index, or
+// just return the answer if it is in the equal section in the middle.
+// The random pivot guarantees average linear time for the same reason that
+// n times vector::push_back takes linear time on average.
+// target_index, start and and end are all indices into the full array.
+// Seed is a seed for rand_r for thread safety purposes. Its value is
+// unimportant as the random numbers do not affect the result except
+// between equal answers.
+template <typename T>
+int GenericVector<T>::choose_nth_item(int target_index, int start, int end,
+                                      unsigned int* seed) {
+  // Number of elements to process.
+  int num_elements = end - start;
+  // Trivial cases.
+  if (num_elements <= 1)
+    return start;
+  if (num_elements == 2) {
+    if (data_[start] < data_[start + 1]) {
+      return target_index > start ? start + 1 : start;
+    } else {
+      return target_index > start ? start : start + 1;
+    }
+  }
+  // Place the pivot at start.
+  #ifndef rand_r  // _MSC_VER, ANDROID
+  srand(*seed);
+  #define rand_r(seed) rand()
+  #endif  // _MSC_VER
+  int pivot = rand_r(seed) % num_elements + start;
+  swap(pivot, start);
+  // The invariant condition here is that items [start, next_lesser) are less
+  // than the pivot (which is at index next_lesser) and items
+  // [prev_greater, end) are greater than the pivot, with items
+  // [next_lesser, prev_greater) being equal to the pivot.
+  int next_lesser = start;
+  int prev_greater = end;
+  for (int next_sample = start + 1; next_sample < prev_greater;) {
+    if (data_[next_sample] < data_[next_lesser]) {
+      swap(next_lesser++, next_sample++);
+    } else if (data_[next_sample] == data_[next_lesser]) {
+      ++next_sample;
+    } else {
+      swap(--prev_greater, next_sample);
+    }
+  }
+  // Now the invariant is set up, we recurse on just the section that contains
+  // the desired index.
+  if (target_index < next_lesser)
+    return choose_nth_item(target_index, start, next_lesser, seed);
+  else if (target_index < prev_greater)
+    return next_lesser;          // In equal bracket.
+  else
+    return choose_nth_item(target_index, prev_greater, end, seed);
+}
+
+
 #endif  // TESSERACT_CCUTIL_GENERICVECTOR_H_
