@@ -17,7 +17,6 @@
  *
  **********************************************************************/
 
-#include "mfcpch.h"
 #include          "fileerr.h"
 #ifdef __UNIX__
 #include          <unistd.h>
@@ -28,7 +27,7 @@
 #include          <stdlib.h>
 #include          "ccutil.h"
 
-#define VARDIR        "configs/" /*variables files */
+#define VARDIR        "configs/" /**< variables files */
 #define EXTERN
 
 const ERRCODE NO_PATH =
@@ -42,39 +41,44 @@ namespace tesseract {
  * Main for mithras demo program. Read the arguments and set up globals.
  **********************************************************************/
 
-void CCUtil::main_setup(                 /*main demo program */
-                const char *argv0,       //program name
-                const char *basename     //name of image
-               ) {
-  imagebasename = basename;      /*name of image */
+/**
+ * @brief CCUtil::main_setup - set location of tessdata and name of image
+ *
+ * @param argv0 - paths to the directory with language files and config files.
+ * An actual value of argv0 is used if not NULL, otherwise TESSDATA_PREFIX is
+ * used if not NULL, next try to use compiled in -DTESSDATA_PREFIX. If previous
+ * is not sucessul - use current directory.
+ * @param basename - name of image
+ */
+void CCUtil::main_setup(const char *argv0, const char *basename) {
+  imagebasename = basename;      /**< name of image */
 
-  // TESSDATA_PREFIX Environment variable overrules everything.
-  // Compiled in -DTESSDATA_PREFIX is next.
-  // An actual value of argv0 is used if not NULL, otherwise current directory.
-  if (!getenv("TESSDATA_PREFIX")) {
+  if (argv0 != NULL) {
+    datadir = argv0;
+  } else {
+    if (getenv("TESSDATA_PREFIX")) {
+      datadir = getenv("TESSDATA_PREFIX");
+    } else {
 #ifdef TESSDATA_PREFIX
 #define _STR(a) #a
 #define _XSTR(a) _STR(a)
     datadir = _XSTR(TESSDATA_PREFIX);
 #undef _XSTR
 #undef _STR
-#else
-    if (argv0 != NULL) {
-      datadir = argv0;
-      // Remove tessdata from the end if present, as we will add it back!
-      int length = datadir.length();
-      if (length >= 8 && strcmp(&datadir[length - 8], "tessdata") == 0)
-        datadir.truncate_at(length - 8);
-      else if (length >= 9 && strcmp(&datadir[length - 9], "tessdata/") == 0)
-        datadir.truncate_at(length - 9);
-      if (datadir.length() == 0)
-        datadir = "./";
-    } else {
-      datadir = "./";
-    }
 #endif
+    }
+  }
+
+  // datadir may still be empty:
+  if (datadir.length() == 0) {
+    datadir = "./";
   } else {
-    datadir = getenv("TESSDATA_PREFIX");
+    // Remove tessdata from the end if present, as we will add it back!
+    int length = datadir.length();
+    if (length >= 8 && strcmp(&datadir[length - 8], "tessdata") == 0)
+      datadir.truncate_at(length - 8);
+    else if (length >= 9 && strcmp(&datadir[length - 9], "tessdata/") == 0)
+      datadir.truncate_at(length - 9);
   }
 
   // check for missing directory separator
@@ -83,6 +87,6 @@ void CCUtil::main_setup(                 /*main demo program */
   if ((strcmp(lastchar, "/") != 0) && (strcmp(lastchar, "\\") != 0))
     datadir += "/";
 
-  datadir += m_data_sub_dir;     /*data directory */
+  datadir += m_data_sub_dir;     /**< data directory */
 }
 }  // namespace tesseract
