@@ -17,7 +17,6 @@
  *
  **********************************************************************/
 
-#include          "mfcpch.h"     // Precompiled headers
 #include          "helpers.h"
 #include          "tprintf.h"
 #include          "strngs.h"
@@ -25,8 +24,11 @@
 
 #include <assert.h>
 // Size of buffer needed to host the decimal representation of the maximum
-// possible length of an int (in 64 bits, being -<20 digits>.
+// possible length of an int (in 64 bits), being -<20 digits>.
 const int kMaxIntSize = 22;
+// Size of buffer needed to host the decimal representation of the maximum
+// possible length of a %.8g being -0.12345678e+999<nul> = 15.
+const int kMaxDoubleSize = 15;
 
 /**********************************************************************
  * STRING_HEADER provides metadata about the allocated buffer,
@@ -96,8 +98,8 @@ void STRING::FixHeader() const {
 
 
 STRING::STRING() {
-  // 0 indicates old NULL -- it doesnt even have '\0'
-  AllocData(0, kMinCapacity);
+  // Empty STRINGs contain just the "\0".
+  memcpy(AllocData(1, kMinCapacity), "", 1);
 }
 
 STRING::STRING(const STRING& str) {
@@ -111,7 +113,8 @@ STRING::STRING(const STRING& str) {
 
 STRING::STRING(const char* cstr) {
   if (cstr == NULL) {
-    AllocData(0, 0);
+    // Empty STRINGs contain just the "\0".
+    memcpy(AllocData(1, kMinCapacity), "", 1);
   } else {
     int len = strlen(cstr) + 1;
     char* this_cstr = AllocData(len, len);
@@ -128,7 +131,7 @@ STRING::~STRING() {
 bool STRING::Serialize(FILE* fp) const {
   inT32 len = length();
   if (fwrite(&len, sizeof(len), 1, fp) != 1) return false;
-  if (fwrite(GetCStr(), 1, len, fp) != len) return false;
+  if (static_cast<int>(fwrite(GetCStr(), 1, len, fp)) != len) return false;
   return true;
 }
 // Reads from the given file. Returns false in case of error.
@@ -139,7 +142,7 @@ bool STRING::DeSerialize(bool swap, FILE* fp) {
   if (swap)
     ReverseN(&len, sizeof(len));
   truncate_at(len);
-  if (fread(GetCStr(), 1, len, fp) != len) return false;
+  if (static_cast<int>(fread(GetCStr(), 1, len, fp)) != len) return false;
   return true;
 }
 
@@ -163,6 +166,10 @@ const char* STRING::string() const {
   return GetCStr();
 }
 
+const char* STRING::c_str() const {
+  return string();
+}
+
 /******
  * The STRING_IS_PROTECTED interface adds additional support to migrate
  * code that needs to modify the STRING in ways not otherwise supported
@@ -220,6 +227,8 @@ void STRING::erase_range(inT32 index, int len) {
 
 #else
 void STRING::truncate_at(inT32 index) {
+  ASSERT_HOST(index >= 0);
+  FixHeader();
   char* this_cstr = ensure_cstr(index + 1);
   this_cstr[index] = '\0';
   GetHeader()->used_ = index + 1;
@@ -339,6 +348,16 @@ void STRING::add_str_int(const char* str, int number) {
   num_buffer[kMaxIntSize - 1] = '\0';
   *this += num_buffer;
 }
+// Appends the given string and double (as a %.8g) to this.
+void STRING::add_str_double(const char* str, double number) {
+  if (str != NULL)
+    *this += str;
+  // Allow space for the maximum possible length of %8g.
+  char num_buffer[kMaxDoubleSize];
+  snprintf(num_buffer, kMaxDoubleSize - 1, "%.8g", number);
+  num_buffer[kMaxDoubleSize - 1] = '\0';
+  *this += num_buffer;
+}
 
 STRING & STRING::operator=(const char* cstr) {
   STRING_HEADER* this_header = GetHeader();
@@ -350,18 +369,29 @@ STRING & STRING::operator=(const char* cstr) {
     this_header = GetHeader();  // for realloc
     memcpy(this_cstr, cstr, len);
     this_header->used_ = len;
-  }
-  else {
-    // Reallocate to zero capacity buffer, consistent with the corresponding
-    // copy constructor.
+  } else {
+    // Reallocate to same state as default constructor.
     DiscardData();
-    AllocData(0, 0);
+    // Empty STRINGs contain just the "\0".
+    memcpy(AllocData(1, kMinCapacity), "", 1);
   }
 
   assert(InvariantOk());
   return *this;
 }
 
+void STRING::assign(const char *cstr, int len) {
+  STRING_HEADER* this_header = GetHeader();
+  this_header->used_ = 0;  // dont bother copying data if need to realloc
+  char* this_cstr = ensure_cstr(len + 1);  // +1 for '\0'
+
+  this_header = GetHeader();  // for realloc
+  memcpy(this_cstr, cstr, len);
+  this_cstr[len] = '\0';
+  this_header->used_ = len + 1;
+
+  assert(InvariantOk());
+}
 
 STRING STRING::operator+(const STRING& str) const {
   STRING result(*this);
