@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 
+#include "helpers.h"
 #include "serialis.h"
 #include "strngs.h"
 #include "tprintf.h"
@@ -35,6 +36,7 @@ namespace tesseract {
 bool TessdataManager::Init(const char *data_file_name, int debug_level) {
   int i;
   debug_level_ = debug_level;
+  data_file_name_ = data_file_name;
   data_file_ = fopen(data_file_name, "rb");
   if (data_file_ == NULL) {
     tprintf("Error opening data file %s\n", data_file_name);
@@ -45,14 +47,15 @@ bool TessdataManager::Init(const char *data_file_name, int debug_level) {
   fread(&actual_tessdata_num_entries_, sizeof(inT32), 1, data_file_);
   swap_ = (actual_tessdata_num_entries_ > kMaxNumTessdataEntries);
   if (swap_) {
-    actual_tessdata_num_entries_ = reverse32(actual_tessdata_num_entries_);
+    ReverseN(&actual_tessdata_num_entries_,
+             sizeof(actual_tessdata_num_entries_));
   }
   ASSERT_HOST(actual_tessdata_num_entries_ <= TESSDATA_NUM_ENTRIES);
   fread(offset_table_, sizeof(inT64),
         actual_tessdata_num_entries_, data_file_);
   if (swap_) {
     for (i = 0 ; i < actual_tessdata_num_entries_; ++i) {
-      offset_table_[i] = reverse64(offset_table_[i]);
+      ReverseN(&offset_table_[i], sizeof(offset_table_[i]));
     }
   }
   if (debug_level_) {
@@ -92,7 +95,9 @@ void TessdataManager::CopyFile(FILE *input_file, FILE *output_file,
   delete[] chunk;
 }
 
-void TessdataManager::WriteMetadata(inT64 *offset_table, FILE *output_file) {
+void TessdataManager::WriteMetadata(inT64 *offset_table,
+                                    const char * language_data_path_prefix,
+                                    FILE *output_file) {
   fseek(output_file, 0, SEEK_SET);
   inT32 num_entries = TESSDATA_NUM_ENTRIES;
   fwrite(&num_entries, sizeof(inT32), 1, output_file);
@@ -101,7 +106,9 @@ void TessdataManager::WriteMetadata(inT64 *offset_table, FILE *output_file) {
 
   tprintf("TessdataManager combined tesseract data files.\n");
   for (int i = 0; i < TESSDATA_NUM_ENTRIES; ++i) {
-    tprintf("Offset for type %d is %lld\n", i, offset_table[i]);
+    tprintf("Offset for type %2d (%s%-22s) is %lld\n", i,
+            language_data_path_prefix, kTessdataFileSuffixes[i],
+            offset_table[i]);
   }
 }
 
@@ -140,20 +147,21 @@ bool TessdataManager::CombineDataFiles(
 
   // Make sure that the required components are present.
   if (file_ptr[TESSDATA_UNICHARSET] == NULL) {
-    tprintf("Error opening unicharset file\n");
+    tprintf("Error opening %sunicharset file\n", language_data_path_prefix);
     fclose(output_file);
     return false;
   }
   if (file_ptr[TESSDATA_INTTEMP] != NULL &&
       (file_ptr[TESSDATA_PFFMTABLE] == NULL ||
        file_ptr[TESSDATA_NORMPROTO] == NULL)) {
-    tprintf("Error opening pffmtable and/or normproto files"
-            " while inttemp file was present\n");
+    tprintf("Error opening %spffmtable and/or %snormproto files"
+            " while %sinttemp file was present\n", language_data_path_prefix,
+            language_data_path_prefix, language_data_path_prefix);
     fclose(output_file);
     return false;
   }
 
-  WriteMetadata(offset_table, output_file);
+  WriteMetadata(offset_table, language_data_path_prefix, output_file);
   return true;
 }
 
@@ -182,8 +190,8 @@ bool TessdataManager::OverwriteComponents(
 
   // Open the files with the new components.
   for (i = 0; i < num_new_components; ++i) {
-    TessdataTypeFromFileName(component_filenames[i], &type, &text_file);
-    file_ptr[type] = fopen(component_filenames[i], "rb");
+    if (TessdataTypeFromFileName(component_filenames[i], &type, &text_file))
+      file_ptr[type] = fopen(component_filenames[i], "rb");
   }
 
   // Write updated data to the output traineddata file.
@@ -203,8 +211,8 @@ bool TessdataManager::OverwriteComponents(
       }
     }
   }
-
-  WriteMetadata(offset_table, output_file);
+  const char *language_data_path_prefix = strchr(new_traineddata_filename, '.');
+  WriteMetadata(offset_table, language_data_path_prefix, output_file);
   return true;
 }
 
@@ -217,7 +225,7 @@ bool TessdataManager::TessdataTypeFromFileSuffix(
       return true;
     }
   }
-  printf("TessdataManager can't determine which tessdata"
+  tprintf("TessdataManager can't determine which tessdata"
          " component is represented by %s\n", suffix);
   return false;
 }
@@ -239,7 +247,7 @@ bool TessdataManager::ExtractToFile(const char *filename) {
 
   FILE *output_file = fopen(filename, "wb");
   if (output_file == NULL) {
-    printf("Error openning %s\n", filename);
+    tprintf("Error opening %s\n", filename);
     exit(1);
   }
   inT64 begin_offset = ftell(GetDataFilePtr());
