@@ -19,7 +19,7 @@
 
 #include <assert.h>
 #include "unichar.h"
-
+#include "host.h"
 #include "unicharmap.h"
 
 UNICHARMAP::UNICHARMAP() :
@@ -99,11 +99,11 @@ void UNICHARMAP::insert(const char* const unichar_repr, UNICHAR_ID id) {
 // string is interpreted as an index in an array of nodes. Stop once the tree
 // does not have anymore nodes or once we found the right unichar_repr.
 bool UNICHARMAP::contains(const char* const unichar_repr) const {
+  if (unichar_repr == NULL || *unichar_repr == '\0') return false;
+
   const char* current_char = unichar_repr;
   UNICHARMAP_NODE* current_nodes = nodes;
 
-  assert(*unichar_repr != '\0');
-
   while (current_nodes != 0 && *(current_char + 1) != '\0') {
     current_nodes =
         current_nodes[static_cast<unsigned char>(*current_char)].children;
@@ -119,12 +119,12 @@ bool UNICHARMAP::contains(const char* const unichar_repr) const {
 // found the right unichar_repr.
 bool UNICHARMAP::contains(const char* const unichar_repr,
                           int length) const {
+  if (unichar_repr == NULL || *unichar_repr == '\0') return false;
+  if (length <= 0 || length > UNICHAR_LEN) return false;
+
   const char* current_char = unichar_repr;
   UNICHARMAP_NODE* current_nodes = nodes;
 
-  assert(*unichar_repr != '\0');
-  assert(length > 0 && length <= UNICHAR_LEN);
-
   while (current_nodes != 0 && (length > 1 && *(current_char + 1) != '\0')) {
     current_nodes =
         current_nodes[static_cast<unsigned char>(*current_char)].children;
@@ -135,6 +135,22 @@ bool UNICHARMAP::contains(const char* const unichar_repr,
       current_nodes[static_cast<unsigned char>(*current_char)].id >= 0;
 }
 
+// Return the minimum number of characters that must be used from this string
+// to obtain a match in the UNICHARMAP.
+int UNICHARMAP::minmatch(const char* const unichar_repr) const {
+  const char* current_char = unichar_repr;
+  UNICHARMAP_NODE* current_nodes = nodes;
+
+  while (current_nodes != NULL && *current_char != '\0') {
+    if (current_nodes[static_cast<unsigned char>(*current_char)].id >= 0)
+      return current_char + 1 - unichar_repr;
+    current_nodes =
+        current_nodes[static_cast<unsigned char>(*current_char)].children;
+    ++current_char;
+  }
+  return 0;
+}
+
 void UNICHARMAP::clear() {
   if (nodes != 0)
   {
