@@ -43,8 +43,10 @@ static const int kBlankFontinfoId = -2;
 
 namespace tesseract {
 
+class ShapeClassifier;
 struct ShapeRating;
 class ShapeTable;
+struct UnicharRating;
 
 // How segmented is a blob. In this enum, character refers to a classifiable
 // unit, but that is too long and character is usually easier to understand.
@@ -67,6 +69,17 @@ class Classify : public CCStruct {
     return shape_table_;
   }
 
+  // Takes ownership of the given classifier, and uses it for future calls
+  // to CharNormClassifier.
+  void SetStaticClassifier(ShapeClassifier* static_classifier);
+
+  // Adds a noise classification result that is a bit worse than the worst
+  // current result, or the worst possible result if no current results.
+  void AddLargeSpeckleTo(int blob_length, BLOB_CHOICE_LIST *choices);
+
+  // Returns true if the blob is small enough to be a large speckle.
+  bool LargeSpeckle(const TBLOB &blob);
+
   /* adaptive.cpp ************************************************************/
   ADAPT_TEMPLATES NewAdaptedTemplates(bool InitFromUnicharset);
   int GetFontinfoId(ADAPT_CLASS Class, uinT8 ConfigId);
@@ -89,7 +102,7 @@ class Classify : public CCStruct {
                    const INT_FEATURE_STRUCT* features,
                    const uinT8* normalization_factors,
                    const uinT16* expected_num_features,
-                   CP_RESULT_STRUCT* results);
+                   GenericVector<CP_RESULT_STRUCT>* results);
   void ReadNewCutoffs(FILE *CutoffFile, bool swap, inT64 end_offset,
                       CLASS_CUTOFF_ARRAY Cutoffs);
   void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates);
@@ -101,7 +114,6 @@ class Classify : public CCStruct {
   void FreeNormProtos();
   NORM_PROTOS *ReadNormProtos(FILE *File, inT64 end_offset);
   /* protos.cpp ***************************************************************/
-  void ReadClassFile();
   void ConvertProto(PROTO Proto, int ProtoId, INT_CLASS Class);
   INT_TEMPLATES CreateIntTemplates(CLASSES FloatProtos,
                                    const UNICHARSET& target_unicharset);
@@ -112,9 +124,7 @@ class Classify : public CCStruct {
   // incorrectly segmented blobs. If filename is not NULL, then LearnBlob
   // is called and the data will be written to a file for static training.
   // Otherwise AdaptToBlob is called for adaption within a document.
-  // If rejmap is not NULL, then only chars with a rejmap entry of '1' will
-  // be learned, otherwise all chars with good correct_text are learned.
-  void LearnWord(const char* filename, const char *rejmap, WERD_RES *word);
+  void LearnWord(const char* filename, WERD_RES *word);
 
   // Builds a blob of length fragments, from the word, starting at start,
   // and then learn it, as having the given correct_text.
@@ -130,31 +140,26 @@ class Classify : public CCStruct {
                    const char* correct_text, WERD_RES *word);
   void InitAdaptiveClassifier(bool load_pre_trained_templates);
   void InitAdaptedClass(TBLOB *Blob,
-                        const DENORM& denorm,
                         CLASS_ID ClassId,
                         int FontinfoId,
                         ADAPT_CLASS Class,
                         ADAPT_TEMPLATES Templates);
-  void AdaptToPunc(TBLOB *Blob,
-                   const DENORM& denorm,
-                   CLASS_ID ClassId,
-                   int FontinfoId,
-                   FLOAT32 Threshold);
-  void AmbigClassifier(TBLOB *Blob,
-                       const DENORM& denorm,
-                       INT_TEMPLATES Templates,
-                       ADAPT_CLASS *Classes,
-                       UNICHAR_ID *Ambiguities,
-                       ADAPT_RESULTS *Results);
+  void AmbigClassifier(const GenericVector<INT_FEATURE_STRUCT>& int_features,
+                       const INT_FX_RESULT_STRUCT& fx_info,
+                       const TBLOB *blob,
+                       INT_TEMPLATES templates,
+                       ADAPT_CLASS *classes,
+                       UNICHAR_ID *ambiguities,
+                       ADAPT_RESULTS *results);
   void MasterMatcher(INT_TEMPLATES templates,
                      inT16 num_features,
                      const INT_FEATURE_STRUCT* features,
                      const uinT8* norm_factors,
                      ADAPT_CLASS* classes,
                      int debug,
-                     int num_classes,
+                     int matcher_multiplier,
                      const TBOX& blob_box,
-                     CLASS_PRUNER_RESULTS results,
+                     const GenericVector<CP_RESULT_STRUCT>& results,
                      ADAPT_RESULTS* final_results);
   // Converts configs to fonts, and if the result is not adapted, and a
   // shape_table_ is present, the shape is expanded to include all
@@ -167,6 +172,7 @@ class Classify : public CCStruct {
                                        int bottom, int top,
                                        float cp_rating,
                                        int blob_length,
+                                       int matcher_multiplier,
                                        const uinT8* cn_factors,
                                        INT_RESULT_STRUCT& int_result,
                                        ADAPT_RESULTS* final_results);
@@ -176,7 +182,8 @@ class Classify : public CCStruct {
   double ComputeCorrectedRating(bool debug, int unichar_id, double cp_rating,
                                 double im_rating, int feature_misses,
                                 int bottom, int top,
-                                int blob_length, const uinT8* cn_factors);
+                                int blob_length, int matcher_multiplier,
+                                const uinT8* cn_factors);
   void ConvertMatchesToChoices(const DENORM& denorm, const TBOX& box,
                                ADAPT_RESULTS *Results,
                                BLOB_CHOICE_LIST *Choices);
@@ -194,15 +201,8 @@ class Classify : public CCStruct {
 
 #ifndef GRAPHICS_DISABLED
   void DebugAdaptiveClassifier(TBLOB *Blob,
-                               const DENORM& denorm,
                                ADAPT_RESULTS *Results);
 #endif
-  void GetAdaptThresholds (TWERD * Word,
-                           const DENORM& denorm,
-                           const WERD_CHOICE& BestChoice,
-                           const WERD_CHOICE& BestRawChoice,
-                           FLOAT32 Thresholds[]);
-
   PROTO_ID MakeNewTempProtos(FEATURE_SET Features,
                              int NumBadFeat,
                              FEATURE_ID BadFeat[],
@@ -218,19 +218,14 @@ class Classify : public CCStruct {
   void MakePermanent(ADAPT_TEMPLATES Templates,
                      CLASS_ID ClassId,
                      int ConfigId,
-                     const DENORM& denorm,
                      TBLOB *Blob);
   void PrintAdaptiveMatchResults(FILE *File, ADAPT_RESULTS *Results);
   void RemoveExtraPuncs(ADAPT_RESULTS *Results);
   void RemoveBadMatches(ADAPT_RESULTS *Results);
   void SetAdaptiveThreshold(FLOAT32 Threshold);
-  void ShowBestMatchFor(TBLOB *Blob,
-                        const DENORM& denorm,
-                        CLASS_ID ClassId,
-                        int shape_id,
-                        BOOL8 AdaptiveOn,
-                        BOOL8 PreTrainedOn,
-                        ADAPT_RESULTS *Results);
+  void ShowBestMatchFor(int shape_id,
+                        const INT_FEATURE_STRUCT* features,
+                        int num_features);
   // Returns a string for the classifier class_id: either the corresponding
   // unicharset debug_str or the shape_table_ debug str.
   STRING ClassIDToDebugStr(const INT_TEMPLATES_STRUCT* templates,
@@ -250,60 +245,38 @@ class Classify : public CCStruct {
   // Converts a shape_table_ index to a classifier class_id index (not a
   // unichar-id!). Uses a search, so not fast.
   int ShapeIDToClassID(int shape_id) const;
-  UNICHAR_ID *BaselineClassifier(TBLOB *Blob,
-                                 const DENORM& denorm,
-                                 ADAPT_TEMPLATES Templates,
-                                 ADAPT_RESULTS *Results);
-  int CharNormClassifier(TBLOB *Blob,
-                         const DENORM& denorm,
-                         INT_TEMPLATES Templates,
-                         ADAPT_RESULTS *Results);
+  UNICHAR_ID *BaselineClassifier(
+      TBLOB *Blob, const GenericVector<INT_FEATURE_STRUCT>& int_features,
+      const INT_FX_RESULT_STRUCT& fx_info,
+      ADAPT_TEMPLATES Templates, ADAPT_RESULTS *Results);
+  int CharNormClassifier(TBLOB *blob,
+                         const TrainingSample& sample,
+                         ADAPT_RESULTS *adapt_results);
 
   // As CharNormClassifier, but operates on a TrainingSample and outputs to
   // a GenericVector of ShapeRating without conversion to classes.
-  int CharNormTrainingSample(bool pruner_only, const TrainingSample& sample,
-                             GenericVector<ShapeRating>* results);
-  UNICHAR_ID *GetAmbiguities(TBLOB *Blob,
-                             const DENORM& denorm,
-                             CLASS_ID CorrectClass);
-  void DoAdaptiveMatch(TBLOB *Blob,
-                       const DENORM& denorm,
-                       ADAPT_RESULTS *Results);
+  int CharNormTrainingSample(bool pruner_only, int keep_this,
+                             const TrainingSample& sample,
+                             GenericVector<UnicharRating>* results);
+  UNICHAR_ID *GetAmbiguities(TBLOB *Blob, CLASS_ID CorrectClass);
+  void DoAdaptiveMatch(TBLOB *Blob, ADAPT_RESULTS *Results);
   void AdaptToChar(TBLOB *Blob,
-                   const DENORM& denorm,
                    CLASS_ID ClassId,
                    int FontinfoId,
                    FLOAT32 Threshold);
-  void DisplayAdaptedChar(TBLOB* blob, const DENORM& denorm,
-                          INT_CLASS_STRUCT* int_class);
-  int AdaptableWord(TWERD *Word,
-                  const WERD_CHOICE &BestChoiceWord,
-                  const WERD_CHOICE &RawChoiceWord);
+  void DisplayAdaptedChar(TBLOB* blob, INT_CLASS_STRUCT* int_class);
+  bool AdaptableWord(WERD_RES* word);
   void EndAdaptiveClassifier();
-  void PrintAdaptiveStatistics(FILE *File);
   void SettupPass1();
   void SettupPass2();
-  void AdaptiveClassifier(TBLOB *Blob,
-                          const DENORM& denorm,
-                          BLOB_CHOICE_LIST *Choices,
-                          CLASS_PRUNER_RESULTS cp_results);
+  void AdaptiveClassifier(TBLOB *Blob, BLOB_CHOICE_LIST *Choices);
   void ClassifyAsNoise(ADAPT_RESULTS *Results);
   void ResetAdaptiveClassifierInternal();
 
-  int GetBaselineFeatures(TBLOB *Blob,
-                          const DENORM& denorm,
-                          INT_TEMPLATES Templates,
-                          INT_FEATURE_ARRAY IntFeatures,
-                          uinT8* CharNormArray,
-                          inT32 *BlobLength);
-  int GetCharNormFeatures(TBLOB *Blob,
-                          const DENORM& denorm,
-                          INT_TEMPLATES Templates,
-                          INT_FEATURE_ARRAY IntFeatures,
-                          uinT8* PrunerNormArray,
-                          uinT8* CharNormArray,
-                          inT32 *BlobLength,
-                          inT32 *FeatureOutlineIndex);
+  int GetCharNormFeature(const INT_FX_RESULT_STRUCT& fx_info,
+                         INT_TEMPLATES templates,
+                         uinT8* pruner_norm_array,
+                         uinT8* char_norm_array);
   // Computes the char_norm_array for the unicharset and, if not NULL, the
   // pruner_array as appropriate according to the existence of the shape_table.
   // The norm_feature is deleted as it is almost certainly no longer needed.
@@ -313,13 +286,53 @@ class Classify : public CCStruct {
                              uinT8* pruner_array);
 
   bool TempConfigReliable(CLASS_ID class_id, const TEMP_CONFIG &config);
-  void UpdateAmbigsGroup(CLASS_ID class_id, const DENORM& denorm, TBLOB *Blob);
+  void UpdateAmbigsGroup(CLASS_ID class_id, TBLOB *Blob);
 
-  void ResetFeaturesHaveBeenExtracted();
   bool AdaptiveClassifierIsFull() { return NumAdaptationsFailed > 0; }
-  bool LooksLikeGarbage(const DENORM& denorm, TBLOB *blob);
+  bool LooksLikeGarbage(TBLOB *blob);
   void RefreshDebugWindow(ScrollView **win, const char *msg,
                           int y_offset, const TBOX &wbox);
+  // intfx.cpp
+  // Computes the DENORMS for bl(baseline) and cn(character) normalization
+  // during feature extraction. The input denorm describes the current state
+  // of the blob, which is usually a baseline-normalized word.
+  // The Transforms setup are as follows:
+  // Baseline Normalized (bl) Output:
+  //   We center the grapheme by aligning the x-coordinate of its centroid with
+  //   x=128 and leaving the already-baseline-normalized y as-is.
+  //
+  // Character Normalized (cn) Output:
+  //   We align the grapheme's centroid at the origin and scale it
+  //   asymmetrically in x and y so that the 2nd moments are a standard value
+  //   (51.2) ie the result is vaguely square.
+  // If classify_nonlinear_norm is true:
+  //   A non-linear normalization is setup that attempts to evenly distribute
+  //   edges across x and y.
+  //
+  // Some of the fields of fx_info are also setup:
+  // Length: Total length of outline.
+  // Rx:     Rounded y second moment. (Reversed by convention.)
+  // Ry:     rounded x second moment.
+  // Xmean:  Rounded x center of mass of the blob.
+  // Ymean:  Rounded y center of mass of the blob.
+  static void SetupBLCNDenorms(const TBLOB& blob, bool nonlinear_norm,
+                               DENORM* bl_denorm, DENORM* cn_denorm,
+                               INT_FX_RESULT_STRUCT* fx_info);
+
+  // Extracts sets of 3-D features of length kStandardFeatureLength (=12.8), as
+  // (x,y) position and angle as measured counterclockwise from the vector
+  // <-1, 0>, from blob using two normalizations defined by bl_denorm and
+  // cn_denorm. See SetpuBLCNDenorms for definitions.
+  // If outline_cn_counts is not NULL, on return it contains the cumulative
+  // number of cn features generated for each outline in the blob (in order).
+  // Thus after the first outline, there were (*outline_cn_counts)[0] features,
+  // after the second outline, there were (*outline_cn_counts)[1] features etc.
+  static void ExtractFeatures(const TBLOB& blob,
+                              bool nonlinear_norm,
+                              GenericVector<INT_FEATURE_STRUCT>* bl_features,
+                              GenericVector<INT_FEATURE_STRUCT>* cn_features,
+                              INT_FX_RESULT_STRUCT* results,
+                              GenericVector<int>* outline_cn_counts);
   /* float2int.cpp ************************************************************/
   void ClearCharNormArray(uinT8* char_norm_array);
   void ComputeIntCharNormArray(const FEATURE_STRUCT& norm_feature,
@@ -336,6 +349,9 @@ class Classify : public CCStruct {
   UnicityTable<FontInfo>& get_fontinfo_table() {
     return fontinfo_table_;
   }
+  const UnicityTable<FontInfo>& get_fontinfo_table() const {
+    return fontinfo_table_;
+  }
   UnicityTable<FontSet>& get_fontset_table() {
     return fontset_table_;
   }
@@ -365,6 +381,10 @@ class Classify : public CCStruct {
   double_VAR_H(classify_max_norm_scale_x, 0.325, "Max char x-norm scale ...");
   double_VAR_H(classify_min_norm_scale_y, 0.0, "Min char y-norm scale ...");
   double_VAR_H(classify_max_norm_scale_y, 0.325, "Max char y-norm scale ...");
+  double_VAR_H(classify_max_rating_ratio, 1.5,
+               "Veto ratio between classifier ratings");
+  double_VAR_H(classify_max_certainty_margin, 5.5,
+               "Veto difference between classifier certainties");
 
   /* adaptmatch.cpp ***********************************************************/
   BOOL_VAR_H(tess_cn_matching, 0, "Character Normalized Matching");
@@ -375,6 +395,8 @@ class Classify : public CCStruct {
   BOOL_VAR_H(classify_save_adapted_templates, 0,
              "Save adapted templates to a file");
   BOOL_VAR_H(classify_enable_adaptive_debugger, 0, "Enable match debugger");
+  BOOL_VAR_H(classify_nonlinear_norm, 0,
+             "Non-linear stroke-density normalization");
   INT_VAR_H(matcher_debug_level, 0, "Matcher Debug Level");
   INT_VAR_H(matcher_debug_flags, 0, "Matcher Debug Flags");
   INT_VAR_H(classify_learning_debug_level, 0, "Learning Debug Level: ");
@@ -398,6 +420,10 @@ class Classify : public CCStruct {
   double_VAR_H(certainty_scale, 20.0, "Certainty scaling factor");
   double_VAR_H(tessedit_class_miss_scale, 0.00390625,
                "Scale factor for features not used");
+  double_VAR_H(classify_adapted_pruning_factor, 2.5,
+               "Prune poor adapted results this much worse than best result");
+  double_VAR_H(classify_adapted_pruning_threshold, -1.0,
+               "Threshold at which classify_adapted_pruning_factor starts");
   INT_VAR_H(classify_adapt_proto_threshold, 230,
             "Threshold for good protos during adaptive 0-255");
   INT_VAR_H(classify_adapt_feature_threshold, 230,
@@ -418,11 +444,11 @@ class Classify : public CCStruct {
   /* intmatcher.cpp **********************************************************/
   INT_VAR_H(classify_class_pruner_threshold, 229,
             "Class Pruner Threshold 0-255");
-  INT_VAR_H(classify_class_pruner_multiplier, 30,
+  INT_VAR_H(classify_class_pruner_multiplier, 15,
             "Class Pruner Multiplier 0-255:       ");
   INT_VAR_H(classify_cp_cutoff_strength, 7,
             "Class Pruner CutoffStrength:         ");
-  INT_VAR_H(classify_integer_matcher_multiplier, 14,
+  INT_VAR_H(classify_integer_matcher_multiplier, 10,
             "Integer Matcher Multiplier  0-255:   ");
 
   // Use class variables to hold onto built-in templates and adapted templates.
@@ -431,9 +457,7 @@ class Classify : public CCStruct {
 
   // Create dummy proto and config masks for use with the built-in templates.
   BIT_VECTOR AllProtosOn;
-  BIT_VECTOR PrunedProtos;
   BIT_VECTOR AllConfigsOn;
-  BIT_VECTOR AllProtosOff;
   BIT_VECTOR AllConfigsOff;
   BIT_VECTOR TempProtoMask;
   bool EnableLearning;
@@ -453,6 +477,9 @@ class Classify : public CCStruct {
   INT_VAR_H(il1_adaption_test, 0, "Dont adapt to i/I at beginning of word");
   BOOL_VAR_H(classify_bln_numeric_mode, 0,
              "Assume the input is numbers [0-9].");
+  double_VAR_H(speckle_large_max_size, 0.30, "Max large speckle size");
+  double_VAR_H(speckle_rating_penalty, 10.0,
+               "Penalty to add to worst rating for noise");
 
  protected:
   IntegerMatcher im_;
@@ -464,32 +491,13 @@ class Classify : public CCStruct {
   ShapeTable* shape_table_;
 
  private:
-
   Dict dict_;
+  // The currently active static classifier.
+  ShapeClassifier* static_classifier_;
 
   /* variables used to hold performance statistics */
-  int AdaptiveMatcherCalls;
-  int BaselineClassifierCalls;
-  int CharNormClassifierCalls;
-  int AmbigClassifierCalls;
-  int NumWordsAdaptedTo;
-  int NumCharsAdaptedTo;
-  int NumBaselineClassesTried;
-  int NumCharNormClassesTried;
-  int NumAmbigClassesTried;
-  int NumClassesOutput;
   int NumAdaptationsFailed;
 
-  /* variables used to hold onto extracted features.  This is used
-  to map from the old scheme in which baseline features and char norm
-  features are extracted separately, to the new scheme in which they
-  are extracted at the same time. */
-  bool FeaturesHaveBeenExtracted;
-  bool FeaturesOK;
-  INT_FEATURE_ARRAY BaselineFeatures;
-  INT_FEATURE_ARRAY CharNormFeatures;
-  INT_FX_RESULT_STRUCT FXInfo;
-
   // Expected number of features in the class pruner, used to penalize
   // unknowns that have too few features (like a c being classified as e) so
   // it doesn't recognize everything as '@' or '#'.
