@@ -224,7 +224,9 @@ void NormalizePicoX(FEATURE_SET FeatureSet) {
 }                                /* NormalizePicoX */
 
 /*---------------------------------------------------------------------------*/
-FEATURE_SET ExtractIntCNFeatures(TBLOB *blob, const DENORM& denorm) {
+FEATURE_SET ExtractIntCNFeatures(TBLOB *blob, const DENORM& bl_denorm,
+                                 const DENORM& cn_denorm,
+                                 const INT_FX_RESULT_STRUCT& fx_info) {
 /*
  ** Parameters:
  **   blob    blob to extract features from
@@ -233,8 +235,11 @@ FEATURE_SET ExtractIntCNFeatures(TBLOB *blob, const DENORM& denorm) {
  ** Exceptions: none
  ** History: 8/8/2011, rays, Created.
  */
-  tesseract::TrainingSample* sample = GetIntFeatures(
-      tesseract::NM_CHAR_ANISOTROPIC, blob, denorm);
+  INT_FX_RESULT_STRUCT local_fx_info(fx_info);
+  GenericVector<INT_FEATURE_STRUCT> bl_features;
+  tesseract::TrainingSample* sample =
+      tesseract::BlobToTrainingSample(*blob, false, &local_fx_info,
+                                      &bl_features);
   if (sample == NULL) return NULL;
 
   int num_features = sample->num_features();
@@ -254,7 +259,9 @@ FEATURE_SET ExtractIntCNFeatures(TBLOB *blob, const DENORM& denorm) {
 }                                /* ExtractIntCNFeatures */
 
 /*---------------------------------------------------------------------------*/
-FEATURE_SET ExtractIntGeoFeatures(TBLOB *blob, const DENORM& denorm) {
+FEATURE_SET ExtractIntGeoFeatures(TBLOB *blob, const DENORM& bl_denorm,
+                                  const DENORM& cn_denorm,
+                                  const INT_FX_RESULT_STRUCT& fx_info) {
 /*
  ** Parameters:
  **   blob    blob to extract features from
@@ -263,8 +270,11 @@ FEATURE_SET ExtractIntGeoFeatures(TBLOB *blob, const DENORM& denorm) {
  ** Exceptions: none
  ** History: 8/8/2011, rays, Created.
  */
-  tesseract::TrainingSample* sample = GetIntFeatures(
-      tesseract::NM_CHAR_ANISOTROPIC, blob, denorm);
+  INT_FX_RESULT_STRUCT local_fx_info(fx_info);
+  GenericVector<INT_FEATURE_STRUCT> bl_features;
+  tesseract::TrainingSample* sample =
+      tesseract::BlobToTrainingSample(*blob, false, &local_fx_info,
+                                      &bl_features);
   if (sample == NULL) return NULL;
 
   FEATURE_SET feature_set = NewFeatureSet(1);
