@@ -22,12 +22,47 @@
 
 #include "shapetable.h"
 
+#include "bitvector.h"
+#include "fontinfo.h"
 #include "intfeaturespace.h"
 #include "strngs.h"
 #include "unicharset.h"
+#include "unicity_table.h"
 
 namespace tesseract {
 
+// Helper function to get the index of the first result with the required
+// unichar_id. If the results are sorted by rating, this will also be the
+// best result with the required unichar_id.
+// Returns -1 if the unichar_id is not found
+int ShapeRating::FirstResultWithUnichar(
+    const GenericVector<ShapeRating>& results,
+    const ShapeTable& shape_table,
+    UNICHAR_ID unichar_id) {
+  for (int r = 0; r < results.size(); ++r) {
+    int shape_id = results[r].shape_id;
+    const Shape& shape = shape_table.GetShape(shape_id);
+    if (shape.ContainsUnichar(unichar_id)) {
+      return r;
+    }
+  }
+  return -1;
+}
+
+// Helper function to get the index of the first result with the required
+// unichar_id. If the results are sorted by rating, this will also be the
+// best result with the required unichar_id.
+// Returns -1 if the unichar_id is not found
+int UnicharRating::FirstResultWithUnichar(
+    const GenericVector<UnicharRating>& results,
+    UNICHAR_ID unichar_id) {
+  for (int r = 0; r < results.size(); ++r) {
+    if (results[r].unichar_id == unichar_id)
+      return r;
+  }
+  return -1;
+}
+
 // Writes to the given file. Returns false in case of error.
 bool UnicharAndFonts::Serialize(FILE* fp) const {
   if (fwrite(&unichar_id, sizeof(unichar_id), 1, fp) != 1) return false;
@@ -138,6 +173,39 @@ bool Shape::ContainsFont(int font_id) const {
   }
   return false;
 }
+// Returns true if the shape contains the given font properties, ignoring
+// unichar_id.
+bool Shape::ContainsFontProperties(const FontInfoTable& font_table,
+                                   uinT32 properties) const {
+  for (int c = 0; c < unichars_.size(); ++c) {
+    GenericVector<int>& font_list = unichars_[c].font_ids;
+    for (int f = 0; f < font_list.size(); ++f) {
+      if (font_table.get(font_list[f]).properties == properties)
+        return true;
+    }
+  }
+  return false;
+}
+// Returns true if the shape contains multiple different font properties,
+// ignoring unichar_id.
+bool Shape::ContainsMultipleFontProperties(
+    const FontInfoTable& font_table) const {
+  uinT32 properties = font_table.get(unichars_[0].font_ids[0]).properties;
+  for (int c = 0; c < unichars_.size(); ++c) {
+    GenericVector<int>& font_list = unichars_[c].font_ids;
+    for (int f = 0; f < font_list.size(); ++f) {
+      if (font_table.get(font_list[f]).properties != properties)
+        return true;
+    }
+  }
+  return false;
+}
+
+// Returns true if this shape is equal to other (ignoring order of unichars
+// and fonts).
+bool Shape::operator==(const Shape& other) const {
+  return IsSubsetOf(other) && other.IsSubsetOf(*this);
+}
 
 // Returns true if this is a subset (including equal) of other.
 bool Shape::IsSubsetOf(const Shape& other) const {
@@ -172,10 +240,10 @@ void Shape::SortUnichars() {
   unichars_sorted_ = true;
 }
 
-ShapeTable::ShapeTable() : unicharset_(NULL) {
+ShapeTable::ShapeTable() : unicharset_(NULL), num_fonts_(0) {
 }
 ShapeTable::ShapeTable(const UNICHARSET& unicharset)
-  : unicharset_(&unicharset) {
+  : unicharset_(&unicharset), num_fonts_(0) {
 }
 
 // Writes to the given file. Returns false in case of error.
@@ -187,9 +255,38 @@ bool ShapeTable::Serialize(FILE* fp) const {
 // If swap is true, assumes a big/little-endian swap is needed.
 bool ShapeTable::DeSerialize(bool swap, FILE* fp) {
   if (!shape_table_.DeSerialize(swap, fp)) return false;
+  num_fonts_ = 0;
   return true;
 }
 
+// Returns the number of fonts used in this ShapeTable, computing it if
+// necessary.
+int ShapeTable::NumFonts() const {
+  if (num_fonts_ <= 0) {
+    for (int shape_id = 0; shape_id < shape_table_.size(); ++shape_id) {
+      const Shape& shape = *shape_table_[shape_id];
+      for (int c = 0; c < shape.size(); ++c) {
+        for (int f = 0; f < shape[c].font_ids.size(); ++f) {
+          if (shape[c].font_ids[f] >= num_fonts_)
+            num_fonts_ = shape[c].font_ids[f] + 1;
+        }
+      }
+    }
+  }
+  return num_fonts_;
+}
+
+// Re-indexes the class_ids in the shapetable according to the given map.
+// Useful in conjunction with set_unicharset.
+void ShapeTable::ReMapClassIds(const GenericVector<int>& unicharset_map) {
+  for (int shape_id = 0; shape_id < shape_table_.size(); ++shape_id) {
+    Shape* shape = shape_table_[shape_id];
+    for (int c = 0; c < shape->size(); ++c) {
+      shape->SetUnicharId(c, unicharset_map[(*shape)[c].unichar_id]);
+    }
+  }
+}
+
 // Returns a string listing the classes/fonts in a shape.
 STRING ShapeTable::DebugStr(int shape_id) const {
   if (shape_id < 0 || shape_id >= shape_table_.size())
@@ -251,15 +348,22 @@ int ShapeTable::AddShape(int unichar_id, int font_id) {
   Shape* shape = new Shape;
   shape->AddToShape(unichar_id, font_id);
   shape_table_.push_back(shape);
+  num_fonts_ = MAX(num_fonts_, font_id + 1);
   return index;
 }
 
-// Adds a copy of the given shape.
-// Returns the assigned index.
+// Adds a copy of the given shape unless it is already present.
+// Returns the assigned index or index of existing shape if already present.
 int ShapeTable::AddShape(const Shape& other) {
-  int index = shape_table_.size();
-  Shape* shape = new Shape(other);
-  shape_table_.push_back(shape);
+  int index;
+  for (index = 0; index < shape_table_.size() &&
+       !(other == *shape_table_[index]); ++index)
+    continue;
+  if (index == shape_table_.size()) {
+    Shape* shape = new Shape(other);
+    shape_table_.push_back(shape);
+  }
+  num_fonts_ = 0;
   return index;
 }
 
@@ -275,12 +379,14 @@ void ShapeTable::DeleteShape(int shape_id) {
 void ShapeTable::AddToShape(int shape_id, int unichar_id, int font_id) {
   Shape& shape = *shape_table_[shape_id];
   shape.AddToShape(unichar_id, font_id);
+  num_fonts_ = MAX(num_fonts_, font_id + 1);
 }
 
 // Adds the given shape to the existing shape with the given index.
 void ShapeTable::AddShapeToShape(int shape_id, const Shape& other) {
   Shape& shape = *shape_table_[shape_id];
   shape.AddShape(other);
+  num_fonts_ = 0;
 }
 
 // Returns the id of the shape that contains the given unichar and font.
@@ -316,25 +422,26 @@ void ShapeTable::GetFirstUnicharAndFont(int shape_id,
 // a ShapeTable.
 int ShapeTable::BuildFromShape(const Shape& shape,
                                const ShapeTable& master_shapes) {
-  int num_masters = 0;
+  BitVector shape_map(master_shapes.NumShapes());
   for (int u_ind = 0; u_ind < shape.size(); ++u_ind) {
     for (int f_ind = 0; f_ind < shape[u_ind].font_ids.size(); ++f_ind) {
       int c = shape[u_ind].unichar_id;
       int f = shape[u_ind].font_ids[f_ind];
-      if (FindShape(c, f) < 0) {
-        int shape_id = AddShape(c, f);
-        int master_id = master_shapes.FindShape(c, f);
-        if (master_id >= 0 && shape.size() > 1) {
-          const Shape& master = master_shapes.GetShape(master_id);
-          if (master.IsSubsetOf(shape) && !shape.IsSubsetOf(master)) {
-            // Add everything else from the master shape.
-            shape_table_[shape_id]->AddShape(master);
-            ++num_masters;
-          }
-        }
+      int master_id = master_shapes.FindShape(c, f);
+      if (master_id >= 0) {
+        shape_map.SetBit(master_id);
+      } else if (FindShape(c, f) < 0) {
+        AddShape(c, f);
       }
     }
   }
+  int num_masters = 0;
+  for (int s = 0; s < master_shapes.NumShapes(); ++s) {
+    if (shape_map[s]) {
+      AddShape(master_shapes.GetShape(s));
+      ++num_masters;
+    }
+  }
   return num_masters;
 }
 
@@ -381,7 +488,7 @@ void ShapeTable::ForceFontMerges(int start, int end) {
     }
   }
   ShapeTable compacted(*unicharset_);
-  compacted.AppendMasterShapes(*this);
+  compacted.AppendMasterShapes(*this, NULL);
   *this = compacted;
 }
 
@@ -422,6 +529,13 @@ void ShapeTable::MergeShapes(int shape_id1, int shape_id2) {
   shape_table_[master_id1]->AddShape(*shape_table_[master_id2]);
 }
 
+// Swaps two shape_ids.
+void ShapeTable::SwapShapes(int shape_id1, int shape_id2) {
+  Shape* tmp = shape_table_[shape_id1];
+  shape_table_[shape_id1] = shape_table_[shape_id2];
+  shape_table_[shape_id2] = tmp;
+}
+
 // Returns the destination of this shape, (if merged), taking into account
 // the fact that the destination may itself have been merged.
 int ShapeTable::MasterDestinationIndex(int shape_id) const {
@@ -435,11 +549,129 @@ int ShapeTable::MasterDestinationIndex(int shape_id) const {
   return master_id;
 }
 
+// Returns false if the unichars in neither shape is a subset of the other.
+bool ShapeTable::SubsetUnichar(int shape_id1, int shape_id2) const {
+  const Shape& shape1 = GetShape(shape_id1);
+  const Shape& shape2 = GetShape(shape_id2);
+  int c1, c2;
+  for (c1 = 0; c1 < shape1.size(); ++c1) {
+    int unichar_id1 = shape1[c1].unichar_id;
+    if (!shape2.ContainsUnichar(unichar_id1))
+      break;
+  }
+  for (c2 = 0; c2 < shape2.size(); ++c2) {
+    int unichar_id2 = shape2[c2].unichar_id;
+    if (!shape1.ContainsUnichar(unichar_id2))
+      break;
+  }
+  return c1 == shape1.size() || c2 == shape2.size();
+}
+
+// Returns false if the unichars in neither shape is a subset of the other.
+bool ShapeTable::MergeSubsetUnichar(int merge_id1, int merge_id2,
+                                    int shape_id) const {
+  const Shape& merge1 = GetShape(merge_id1);
+  const Shape& merge2 = GetShape(merge_id2);
+  const Shape& shape = GetShape(shape_id);
+  int cm1, cm2, cs;
+  for (cs = 0; cs < shape.size(); ++cs) {
+    int unichar_id = shape[cs].unichar_id;
+    if (!merge1.ContainsUnichar(unichar_id) &&
+        !merge2.ContainsUnichar(unichar_id))
+      break;  // Shape is not a subset of the merge.
+  }
+  for (cm1 = 0; cm1 < merge1.size(); ++cm1) {
+    int unichar_id1 = merge1[cm1].unichar_id;
+    if (!shape.ContainsUnichar(unichar_id1))
+      break;  // Merge is not a subset of shape
+  }
+  for (cm2 = 0; cm2 < merge2.size(); ++cm2) {
+    int unichar_id2 = merge2[cm2].unichar_id;
+    if (!shape.ContainsUnichar(unichar_id2))
+      break;  // Merge is not a subset of shape
+  }
+  return cs == shape.size() || (cm1 == merge1.size() && cm2 == merge2.size());
+}
+
+// Returns true if the unichar sets are equal between the shapes.
+bool ShapeTable::EqualUnichars(int shape_id1, int shape_id2) const {
+  const Shape& shape1 = GetShape(shape_id1);
+  const Shape& shape2 = GetShape(shape_id2);
+  for (int c1 = 0; c1 < shape1.size(); ++c1) {
+    int unichar_id1 = shape1[c1].unichar_id;
+    if (!shape2.ContainsUnichar(unichar_id1))
+      return false;
+  }
+  for (int c2 = 0; c2 < shape2.size(); ++c2) {
+    int unichar_id2 = shape2[c2].unichar_id;
+    if (!shape1.ContainsUnichar(unichar_id2))
+      return false;
+  }
+  return true;
+}
+
+// Returns true if the unichar sets are equal between the shapes.
+bool ShapeTable::MergeEqualUnichars(int merge_id1, int merge_id2,
+                                    int shape_id) const {
+  const Shape& merge1 = GetShape(merge_id1);
+  const Shape& merge2 = GetShape(merge_id2);
+  const Shape& shape = GetShape(shape_id);
+  for (int cs = 0; cs < shape.size(); ++cs) {
+    int unichar_id = shape[cs].unichar_id;
+    if (!merge1.ContainsUnichar(unichar_id) &&
+        !merge2.ContainsUnichar(unichar_id))
+      return false;  // Shape has a unichar that appears in neither merge.
+  }
+  for (int cm1 = 0; cm1 < merge1.size(); ++cm1) {
+    int unichar_id1 = merge1[cm1].unichar_id;
+    if (!shape.ContainsUnichar(unichar_id1))
+      return false;  // Merge has a unichar that is not in shape.
+  }
+  for (int cm2 = 0; cm2 < merge2.size(); ++cm2) {
+    int unichar_id2 = merge2[cm2].unichar_id;
+    if (!shape.ContainsUnichar(unichar_id2))
+      return false;  // Merge has a unichar that is not in shape.
+  }
+  return true;
+}
+
+// Returns true if there is a common unichar between the shapes.
+bool ShapeTable::CommonUnichars(int shape_id1, int shape_id2) const {
+  const Shape& shape1 = GetShape(shape_id1);
+  const Shape& shape2 = GetShape(shape_id2);
+  for (int c1 = 0; c1 < shape1.size(); ++c1) {
+    int unichar_id1 = shape1[c1].unichar_id;
+    if (shape2.ContainsUnichar(unichar_id1))
+      return true;
+  }
+  return false;
+}
+
+// Returns true if there is a common font id between the shapes.
+bool ShapeTable::CommonFont(int shape_id1, int shape_id2) const {
+  const Shape& shape1 = GetShape(shape_id1);
+  const Shape& shape2 = GetShape(shape_id2);
+  for (int c1 = 0; c1 < shape1.size(); ++c1) {
+    const GenericVector<int>& font_list1 = shape1[c1].font_ids;
+    for (int f = 0; f < font_list1.size(); ++f) {
+      if (shape2.ContainsFont(font_list1[f]))
+        return true;
+    }
+  }
+  return false;
+}
+
 // Appends the master shapes from other to this.
-void ShapeTable::AppendMasterShapes(const ShapeTable& other) {
+// If not NULL, shape_map is set to map other shape_ids to this's shape_ids.
+void ShapeTable::AppendMasterShapes(const ShapeTable& other,
+                                    GenericVector<int>* shape_map) {
+  if (shape_map != NULL)
+    shape_map->init_to_size(other.NumShapes(), -1);
   for (int s = 0; s < other.shape_table_.size(); ++s) {
     if (other.shape_table_[s]->destination_index() < 0) {
-      AddShape(*other.shape_table_[s]);
+      int index = AddShape(*other.shape_table_[s]);
+      if (shape_map != NULL)
+        (*shape_map)[s] = index;
     }
   }
 }
@@ -455,6 +687,46 @@ int ShapeTable::NumMasterShapes() const {
 }
 
 
+// Adds the unichars of the given shape_id to the vector of results. Any
+// unichar_id that is already present just has the fonts added to the
+// font set for that result without adding a new entry in the vector.
+// NOTE: it is assumed that the results are given to this function in order
+// of decreasing rating.
+// The unichar_map vector indicates the index of the results entry containing
+// each unichar, or -1 if the unichar is not yet included in results.
+void ShapeTable::AddShapeToResults(const ShapeRating& shape_rating,
+                                   GenericVector<int>* unichar_map,
+                                   GenericVector<UnicharRating>* results)const {
+  if (shape_rating.joined) {
+    AddUnicharToResults(UNICHAR_JOINED, shape_rating.rating, unichar_map,
+                        results);
+  }
+  if (shape_rating.broken) {
+    AddUnicharToResults(UNICHAR_BROKEN, shape_rating.rating, unichar_map,
+                        results);
+  }
+  const Shape& shape = GetShape(shape_rating.shape_id);
+  for (int u = 0; u < shape.size(); ++u) {
+    int result_index = AddUnicharToResults(shape[u].unichar_id,
+                                           shape_rating.rating,
+                                           unichar_map, results);
+    (*results)[result_index].fonts += shape[u].font_ids;
+  }
+}
+
+// Adds the given unichar_id to the results if needed, updating unichar_map
+// and returning the index of unichar in results.
+int ShapeTable::AddUnicharToResults(
+    int unichar_id, float rating, GenericVector<int>* unichar_map,
+    GenericVector<UnicharRating>* results) const {
+  int result_index = unichar_map->get(unichar_id);
+  if (result_index < 0) {
+    UnicharRating result(unichar_id, rating);
+    result_index = results->push_back(result);
+    (*unichar_map)[unichar_id] = result_index;
+  }
+  return result_index;
+}
+
+
 }  // namespace tesseract
-
-
