@@ -88,52 +88,6 @@ char_32 *CubeUtils::StrDup(const char_32 *str32) {
   return new_str;
 }
 
-// creates a raw buffer from the specified location of the image
-unsigned char *CubeUtils::GetImageData(IMAGE *img, int left,
-                                       int top, int wid, int hgt) {
-  // skip invalid dimensions
-  if (left < 0 || top < 0 || wid < 0 || hgt < 0 ||
-      (left + wid) > img->get_xsize() ||
-      (top + hgt) > img->get_ysize()) {
-    return NULL;
-  }
-
-  // copy the char img to a temp buffer
-  unsigned char *temp_buff = new unsigned char[wid * hgt];
-  if (temp_buff == NULL) {
-    return NULL;
-  }
-
-  IMAGELINE line;
-  line.init(wid);
-
-  for (int y = 0, off = 0; y < hgt ; y++) {
-    img->get_line(left, img->get_ysize() - 1 - y - top, wid, &line, 0);
-    for (int x = 0; x < wid; x++, off++) {
-      temp_buff[off] = line.pixels[x] ? 255 : 0;
-    }
-  }
-
-  return temp_buff;
-}
-
-// creates a char samp from a specified portion of the image
-CharSamp *CubeUtils::CharSampleFromImg(IMAGE *img,
-                                       int left, int top,
-                                       int wid, int hgt) {
-  // get the raw img data from the image
-  unsigned char *temp_buff = GetImageData(img, left, top, wid, hgt);
-  if (temp_buff == NULL) {
-    return NULL;
-  }
-
-  // create a char samp from temp buffer
-  CharSamp *char_samp = CharSamp::FromRawData(left, top, wid, hgt, temp_buff);
-  // clean up temp buffer
-  delete []temp_buff;
-  return char_samp;
-}
-
 // creates a char samp from a specified portion of the image
 CharSamp *CubeUtils::CharSampleFromPix(Pix *pix, int left, int top,
                                        int wid, int hgt) {
@@ -151,50 +105,6 @@ CharSamp *CubeUtils::CharSampleFromPix(Pix *pix, int left, int top,
   return char_samp;
 }
 
-// create a B/W image from a char_sample
-IMAGE *CubeUtils::ImageFromCharSample(CharSamp *char_samp) {
-  // parameter check
-  if (char_samp == NULL) {
-    return NULL;
-  }
-
-  // get the raw data
-  int stride = char_samp->Stride(),
-    wid = char_samp->Width(),
-    hgt = char_samp->Height();
-
-  unsigned char  *buff = char_samp->RawData();
-  if (buff == NULL) {
-    return NULL;
-  }
-
-  // create a new image object
-  IMAGE *img = new IMAGE();
-  if (img == NULL) {
-    return NULL;
-  }
-
-  // create a blank B/W image
-  if (img->create(wid, hgt, 1) == -1) {
-    delete img;
-    return NULL;
-  }
-
-  // copy the contents
-  IMAGELINE line;
-  line.init(wid);
-
-  for (int y = 0, off = 0; y < hgt ; y++, off += stride) {
-    for (int x = 0; x < wid; x++) {
-      line.pixels[x] = (buff[off + x] == 0) ? 0 : 1;
-    }
-
-    img->fast_put_line(0, hgt - 1 - y, wid, &line);
-  }
-
-  return img;
-}
-
 // create a B/W image from a char_sample
 Pix *CubeUtils::PixFromCharSample(CharSamp *char_samp) {
   // parameter check
@@ -242,7 +152,6 @@ unsigned char *CubeUtils::GetImageData(Pix *pix, int left, int top,
   if (temp_buff == NULL) {
     return NULL;
   }
-
   l_int32 w;
   l_int32 h;
   l_int32 d;
