@@ -29,12 +29,6 @@
 #include "const.h"
 #include "char_samp.h"
 
-#ifdef _WIN32
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-#endif
-
 namespace tesseract {
 
 FeatureChebyshev::FeatureChebyshev(TuningParams *params)
