@@ -53,7 +53,6 @@ bool WordListLangModel::Init() {
   // The last parameter to the Trie constructor (the debug level) is set to
   // false for now, until Cube has a way to express its preferred debug level.
   dawg_ = new Trie(DAWG_TYPE_WORD, "", NO_PERM,
-                   WordListLangModel::kMaxDawgEdges,
                    cntxt_->CharacterSet()->ClassCount(), false);
   if (dawg_ == NULL) {
     return false;
@@ -74,7 +73,7 @@ LangModEdge **WordListLangModel::GetEdges(CharAltList *alt_list,
   // initialize if necessary
   if (init_ == false) {
     if (Init() == false) {
-      return false;
+      return NULL;
     }
   }
 
@@ -92,7 +91,7 @@ LangModEdge **WordListLangModel::GetEdges(CharAltList *alt_list,
     // advance node
     edge_ref = dawg_->next_node(edge_ref);
     if (edge_ref == 0) {
-      return 0;
+      return NULL;
     }
   }
 
