@@ -17,6 +17,10 @@
 //
 ///////////////////////////////////////////////////////////////////////
 
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif
+
 #include "ccnontextdetect.h"
 #include "imagefind.h"
 #include "strokewidth.h"
