@@ -22,16 +22,17 @@
 #pragma warning(disable:4244)  // Conversion warnings
 #endif
 
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif
+
 #include "linefind.h"
 #include "alignedblob.h"
 #include "tabvector.h"
 #include "blobbox.h"
 #include "edgblob.h"
-// This entire file is dependent upon leptonica. If you don't have it,
-// then the code doesn't do anything useful.
-#ifdef HAVE_CONFIG_H
-#include "config_auto.h"
-#endif
+#include "openclwrapper.h"
+
 #include "allheaders.h"
 
 namespace tesseract {
@@ -69,7 +70,7 @@ static void RemoveUnusedLineSegments(bool horizontal_lines,
   BLOBNBOX_IT bbox_it(line_bblobs);
   for (bbox_it.mark_cycle_pt(); !bbox_it.cycled_list(); bbox_it.forward()) {
     BLOBNBOX* blob = bbox_it.data();
-    if (blob->left_tab_type() == TT_MAYBE_ALIGNED) {
+    if (blob->left_tab_type() != TT_VLINE) {
       const TBOX& box = blob->bounding_box();
       Box* pixbox = NULL;
       if (horizontal_lines) {
@@ -79,7 +80,6 @@ static void RemoveUnusedLineSegments(bool horizontal_lines,
         // See GetLineBoxes for more explanation.
         pixbox = boxCreate(box.bottom(), height - box.right(),
                            box.height(), box.width());
-
       } else {
         // For vertical lines, just flip upside-down to convert to Leptonica.
         // The y position of the box in Leptonica terms is the distance from
@@ -245,6 +245,7 @@ void LineFinder::FindAndRemoveLines(int resolution, bool debug, Pix* pix,
                                     Pix** pix_music_mask,
                                     TabVector_LIST* v_lines,
                                     TabVector_LIST* h_lines) {
+  PERF_COUNT_START("FindAndRemoveLines")
   if (pix == NULL || vertical_x == NULL || vertical_y == NULL) {
     tprintf("Error in parameters for LineFinder::FindAndRemoveLines\n");
     return;
@@ -311,6 +312,7 @@ void LineFinder::FindAndRemoveLines(int resolution, bool debug, Pix* pix,
 #endif
     pixaDestroy(&pixa_display);
   }
+  PERF_COUNT_END
 }
 
 // Converts the Boxa array to a list of C_BLOB, getting rid of severely
@@ -574,6 +576,9 @@ void LineFinder::GetLineMasks(int resolution, Pix* src_pix,
                               Pix** pix_hline, Pix** pix_non_hline,
                               Pix** pix_intersections, Pix** pix_music_mask,
                               Pixa* pixa_display) {
+  Pix* pix_closed = NULL;
+  Pix* pix_hollow = NULL;
+
   int max_line_width = resolution / kThinLineFraction;
   int min_line_length = resolution / kMinLineLengthFraction;
   if (pixa_display != NULL) {
@@ -582,10 +587,25 @@ void LineFinder::GetLineMasks(int resolution, Pix* src_pix,
   }
   int closing_brick = max_line_width / 3;
 
+  PERF_COUNT_START("GetLineMasksMorph")
+// only use opencl if compiled w/ OpenCL and selected device is opencl
+#ifdef USE_OPENCL
+  if (OpenclDevice::selectedDeviceIsOpenCL()) {
+    // OpenCL pixGetLines Operation
+    int clStatus = OpenclDevice::initMorphCLAllocations(pixGetWpl(src_pix),
+                                                        pixGetHeight(src_pix),
+                                                        src_pix);
+    bool getpixclosed = pix_music_mask != NULL ? true : false;
+    OpenclDevice::pixGetLinesCL(NULL, src_pix, pix_vline, pix_hline,
+                                &pix_closed, getpixclosed, closing_brick,
+                                closing_brick, max_line_width, max_line_width,
+                                min_line_length, min_line_length);
+  } else {
+#endif
   // Close up small holes, making it less likely that false alarms are found
   // in thickened text (as it will become more solid) and also smoothing over
   // some line breaks and nicks in the edges of the lines.
-  Pix* pix_closed = pixCloseBrick(NULL, src_pix, closing_brick, closing_brick);
+  pix_closed = pixCloseBrick(NULL, src_pix, closing_brick, closing_brick);
   if (pixa_display != NULL)
     pixaAddPix(pixa_display, pix_closed, L_CLONE);
   // Open up with a big box to detect solid areas, which can then be subtracted.
@@ -594,15 +614,23 @@ void LineFinder::GetLineMasks(int resolution, Pix* src_pix,
                                 max_line_width);
   if (pixa_display != NULL)
     pixaAddPix(pixa_display, pix_solid, L_CLONE);
-  Pix* pix_hollow = pixSubtract(NULL, pix_closed, pix_solid);
+  pix_hollow = pixSubtract(NULL, pix_closed, pix_solid);
+
   pixDestroy(&pix_solid);
+
   // Now open up in both directions independently to find lines of at least
   // 1 inch/kMinLineLengthFraction in length.
   if (pixa_display != NULL)
     pixaAddPix(pixa_display, pix_hollow, L_CLONE);
   *pix_vline = pixOpenBrick(NULL, pix_hollow, 1, min_line_length);
   *pix_hline = pixOpenBrick(NULL, pix_hollow, min_line_length, 1);
+
   pixDestroy(&pix_hollow);
+#ifdef USE_OPENCL
+  }
+#endif
+  PERF_COUNT_END
+
   // Lines are sufficiently rare, that it is worth checking for a zero image.
   l_int32 v_empty = 0;
   l_int32 h_empty = 0;
