@@ -21,12 +21,13 @@
 #pragma warning(disable:4244)  // Conversion warnings
 #endif
 
-#include "tablefind.h"
-#include <math.h>
 #ifdef HAVE_CONFIG_H
 #include "config_auto.h"
 #endif
 
+#include "tablefind.h"
+#include <math.h>
+
 #include "allheaders.h"
 
 #include "colpartitionset.h"
@@ -271,6 +272,8 @@ void TableFinder::LocateTables(ColPartitionGrid* grid,
                                const FCOORD& reskew) {
   // initialize spacing, neighbors, and columns
   InitializePartitions(all_columns);
+
+#ifndef GRAPHICS_DISABLED
   if (textord_show_tables) {
     ScrollView* table_win = MakeWindow(0, 300, "Column Partitions & Neighbors");
     DisplayColPartitions(table_win, &clean_part_grid_, ScrollView::BLUE);
@@ -282,6 +285,8 @@ void TableFinder::LocateTables(ColPartitionGrid* grid,
     table_win = MakeWindow(100, 300, "Fragmented Text");
     DisplayColPartitions(table_win, &fragmented_text_grid_, ScrollView::BLUE);
   }
+#endif  // GRAPHICS_DISABLED
+
   // mark, filter, and smooth candidate table partitions
   MarkTablePartitions();
 
@@ -310,11 +315,13 @@ void TableFinder::LocateTables(ColPartitionGrid* grid,
   ColSegment_LIST table_regions;
   GetTableRegions(&table_columns, &table_regions);
 
+#ifndef GRAPHICS_DISABLED
   if (textord_tablefind_show_mark) {
     ScrollView* table_win = MakeWindow(1200, 300, "Table Columns and Regions");
     DisplayColSegments(table_win, &table_columns, ScrollView::DARK_TURQUOISE);
     DisplayColSegments(table_win, &table_regions, ScrollView::YELLOW);
   }
+#endif  // GRAPHICS_DISABLED
 
   // Merge table regions across columns for tables spanning multiple
   // columns
@@ -330,24 +337,28 @@ void TableFinder::LocateTables(ColPartitionGrid* grid,
     // Remove false alarms consiting of a single column
     DeleteSingleColumnTables();
 
+#ifndef GRAPHICS_DISABLED
     if (textord_show_tables) {
       ScrollView* table_win = MakeWindow(1200, 300, "Detected Table Locations");
       DisplayColPartitions(table_win, &clean_part_grid_, ScrollView::BLUE);
       DisplayColSegments(table_win, &table_columns, ScrollView::KHAKI);
       table_grid_.DisplayBoxes(table_win);
     }
+#endif  // GRAPHICS_DISABLED
 
     // Find table grid structure and reject tables that are malformed.
     RecognizeTables();
     GridMergeTableRegions();
     RecognizeTables();
 
+#ifndef GRAPHICS_DISABLED
     if (textord_show_tables) {
       ScrollView* table_win = MakeWindow(1400, 600, "Recognized Tables");
       DisplayColPartitions(table_win, &clean_part_grid_,
                            ScrollView::BLUE, ScrollView::BLUE);
       table_grid_.DisplayBoxes(table_win);
     }
+#endif  // GRAPHICS_DISABLED
   } else {
     // Remove false alarms consiting of a single column
     // TODO(nbeato): verify this is a NOP after structured table rejection.
@@ -355,12 +366,14 @@ void TableFinder::LocateTables(ColPartitionGrid* grid,
     // supposed to do, this function is obsolete.
     DeleteSingleColumnTables();
 
+#ifndef GRAPHICS_DISABLED
     if (textord_show_tables) {
       ScrollView* table_win = MakeWindow(1500, 300, "Detected Tables");
       DisplayColPartitions(table_win, &clean_part_grid_,
                            ScrollView::BLUE, ScrollView::BLUE);
       table_grid_.DisplayBoxes(table_win);
     }
+#endif  // GRAPHICS_DISABLED
   }
 
   if (textord_dump_table_images)
@@ -1457,7 +1470,8 @@ bool TableFinder::BelongToOneTable(const TBOX &box1, const TBOX &box2) {
   while ((part = rectsearch.NextRectSearch()) != NULL) {
     const TBOX& part_box = part->bounding_box();
     // return true if a colpartition spanning both table regions is found
-    if (part_box.overlap(box1) && part_box.overlap(box2))
+    if (part_box.overlap(box1) && part_box.overlap(box2) &&
+        !part->IsImageType())
       return true;
   }
   return false;
