@@ -27,16 +27,19 @@
 
 #include "host.h"
 #include "callcpp.h"
-#include "tessclas.h"
+#include "blobs.h"
 
 /*----------------------------------------------------------------------
               V a r i a b l e s
 ----------------------------------------------------------------------*/
 extern ScrollView *blob_window;        /* Window for blobs */
 extern C_COL color_list[];       /* Colors for outlines */
-extern int blob_pause;           /* Wait after each blob */
-extern int display_all_blobs;    /* Display blobs ? */
-extern int display_all_words;    /* Display words ? */
+
+extern BOOL_VAR_H(wordrec_display_all_blobs, 0, "Display Blobs");
+
+extern BOOL_VAR_H(wordrec_display_all_words, 0, "Display Words");
+
+extern BOOL_VAR_H(wordrec_blob_pause, 0, "Blob pause");
 
 #define NUM_COLORS 6
 
@@ -45,8 +48,6 @@ extern int display_all_words;    /* Display words ? */
 ----------------------------------------------------------------------*/
 void display_blob(TBLOB *blob, C_COL color);
 
-void init_render_vars();
-
 void render_blob(void *window, TBLOB *blob, C_COL color);
 
 void render_edgepts(void *window, EDGEPT *edgept, C_COL color);
